@@ -54,6 +54,16 @@ if (!function_exists('json_encode') || !function_exists('json_decode')) {
 
 define('NO_OUTPUT_BUFFERING', true);
 
+if (isset($_POST['upgradekey'])) {
+    // Before you start reporting issues about the collision attacks against
+    // SHA-1, you should understand that we are not actually attempting to do
+    // any cryptography here. This is hashed purely so that the key is not
+    // that apparent in the address bar itself. Anyone who catches the HTTP
+    // traffic can immediately use it as a valid admin key.
+    header('Location: index.php?cache=0&upgradekeyhash='.sha1($_POST['upgradekey']));
+    die();
+}
+
 if ((isset($_GET['cache']) and $_GET['cache'] === '0')
         or (isset($_POST['cache']) and $_POST['cache'] === '0')
         or (!isset($_POST['cache']) and !isset($_GET['cache']) and empty($_GET['sesskey']) and empty($_POST['sesskey']))) {
@@ -88,23 +98,51 @@ core_component::get_core_subsystems();
 require_once($CFG->libdir.'/adminlib.php');    // various admin-only functions
 require_once($CFG->libdir.'/upgradelib.php');  // general upgrade/install related functions
 
-$id             = optional_param('id', '', PARAM_TEXT);
-$confirmupgrade = optional_param('confirmupgrade', 0, PARAM_BOOL);
-$confirmrelease = optional_param('confirmrelease', 0, PARAM_BOOL);
-$confirmplugins = optional_param('confirmplugincheck', 0, PARAM_BOOL);
-$showallplugins = optional_param('showallplugins', 0, PARAM_BOOL);
-$agreelicense   = optional_param('agreelicense', 0, PARAM_BOOL);
-$fetchupdates   = optional_param('fetchupdates', 0, PARAM_BOOL);
-$newaddonreq    = optional_param('installaddonrequest', null, PARAM_RAW);
+$confirmupgrade = optional_param('confirmupgrade', 0, PARAM_BOOL); // Core upgrade confirmed?
+$confirmrelease = optional_param('confirmrelease', 0, PARAM_BOOL); // Core release info and server checks confirmed?
+$confirmplugins = optional_param('confirmplugincheck', 0, PARAM_BOOL); // Plugins check page confirmed?
+$showallplugins = optional_param('showallplugins', 0, PARAM_BOOL); // Show all plugins on the plugins check page?
+$agreelicense = optional_param('agreelicense', 0, PARAM_BOOL); // GPL license confirmed for installation?
+$fetchupdates = optional_param('fetchupdates', 0, PARAM_BOOL); // Should check for available updates?
+$newaddonreq = optional_param('installaddonrequest', null, PARAM_RAW); // Plugin installation requested at moodle.org/plugins.
+$upgradekeyhash = optional_param('upgradekeyhash', null, PARAM_ALPHANUM); // Hash of provided upgrade key.
+$installdep = optional_param('installdep', null, PARAM_COMPONENT); // Install given missing dependency (required plugin).
+$installdepx = optional_param('installdepx', false, PARAM_BOOL); // Install all missing dependencies.
+$confirminstalldep = optional_param('confirminstalldep', false, PARAM_BOOL); // Installing dependencies confirmed.
+$abortinstall = optional_param('abortinstall', null, PARAM_COMPONENT); // Cancel installation of the given new plugin.
+$abortinstallx = optional_param('abortinstallx', null, PARAM_BOOL); // Cancel installation of all new plugins.
+$confirmabortinstall = optional_param('confirmabortinstall', false, PARAM_BOOL); // Installation cancel confirmed.
+$abortupgrade = optional_param('abortupgrade', null, PARAM_COMPONENT); // Cancel upgrade of the given existing plugin.
+$abortupgradex = optional_param('abortupgradex', null, PARAM_BOOL); // Cancel upgrade of all upgradable plugins.
+$confirmabortupgrade = optional_param('confirmabortupgrade', false, PARAM_BOOL); // Upgrade cancel confirmed.
+$installupdate = optional_param('installupdate', null, PARAM_COMPONENT); // Install given available update.
+$installupdateversion = optional_param('installupdateversion', null, PARAM_INT); // Version of the available update to install.
+$installupdatex = optional_param('installupdatex', false, PARAM_BOOL); // Install all available plugin updates.
+$confirminstallupdate = optional_param('confirminstallupdate', false, PARAM_BOOL); // Available update(s) install confirmed?
+
+if (!empty($CFG->disableupdateautodeploy)) {
+    // Invalidate all requests to install plugins via the admin UI.
+    $newaddonreq = null;
+    $installdep = null;
+    $installdepx = false;
+    $abortupgrade = null;
+    $abortupgradex = null;
+    $installupdate = null;
+    $installupdateversion = null;
+    $installupdatex = false;
+}
 
 // Set up PAGE.
 $url = new moodle_url('/admin/index.php');
 $url->param('cache', $cache);
+if (isset($upgradekeyhash)) {
+    $url->param('upgradekeyhash', $upgradekeyhash);
+}
 $PAGE->set_url($url);
 unset($url);
 
 // Are we returning from an add-on installation request at moodle.org/plugins?
-if ($newaddonreq and !$cache and empty($CFG->disableonclickaddoninstall)) {
+if ($newaddonreq and !$cache and empty($CFG->disableupdateautodeploy)) {
     $target = new moodle_url('/admin/tool/installaddon/index.php', array(
         'installaddonrequest' => $newaddonreq,
         'confirm' => 0));
@@ -175,7 +213,6 @@ if (!core_tables_exist()) {
         $PAGE->set_heading($strinstallation);
         $PAGE->set_cacheable(false);
 
-        /** @var core_admin_renderer $output */
         $output = $PAGE->get_renderer('core', 'admin');
         echo $output->install_licence_page();
         die();
@@ -190,7 +227,6 @@ if (!core_tables_exist()) {
         $PAGE->set_heading($strinstallation . ' - Moodle ' . $CFG->target_release);
         $PAGE->set_cacheable(false);
 
-        /** @var core_admin_renderer $output */
         $output = $PAGE->get_renderer('core', 'admin');
         echo $output->install_environment_page($maturity, $envstatus, $environment_results, $release);
         die();
@@ -204,7 +240,7 @@ if (!core_tables_exist()) {
         $PAGE->set_heading($strinstallation . ' - Moodle ' . $CFG->target_release);
 
         $output = $PAGE->get_renderer('core', 'admin');
-        $url = new moodle_url('/admin/index.php', array('agreelicense' => 1, 'confirmrelease' => 1, 'lang' => $CFG->lang));
+        $url = new moodle_url($PAGE->url, array('agreelicense' => 1, 'confirmrelease' => 1, 'lang' => $CFG->lang));
         echo $output->unsatisfied_dependencies_page($version, $failed, $url);
         die();
     }
@@ -254,11 +290,19 @@ if (empty($CFG->version)) {
 // Detect config cache inconsistency, this happens when you switch branches on dev servers.
 if ($CFG->version != $DB->get_field('config', 'value', array('name'=>'version'))) {
     purge_all_caches();
-    redirect(new moodle_url('/admin/index.php'), 'Config cache inconsistency detected, resetting caches...');
+    redirect(new moodle_url($PAGE->url), 'Config cache inconsistency detected, resetting caches...');
 }
 
 if (!$cache and $version > $CFG->version) {  // upgrade
 
+    $PAGE->set_url(new moodle_url($PAGE->url, array(
+        'confirmupgrade' => $confirmupgrade,
+        'confirmrelease' => $confirmrelease,
+        'confirmplugincheck' => $confirmplugins,
+    )));
+
+    check_upgrade_key($upgradekeyhash);
+
     // Warning about upgrading a test site.
     $testsite = false;
     if (defined('BEHAT_SITE_RUNNING')) {
@@ -274,7 +318,6 @@ if (!$cache and $version > $CFG->version) {  // upgrade
     // We then purge the regular caches.
     purge_all_caches();
 
-    /** @var core_admin_renderer $output */
     $output = $PAGE->get_renderer('core', 'admin');
 
     if (upgrade_stale_php_files_present()) {
@@ -319,40 +362,137 @@ if (!$cache and $version > $CFG->version) {  // upgrade
         $PAGE->set_heading($strplugincheck);
         $PAGE->set_cacheable(false);
 
-        $reloadurl = new moodle_url('/admin/index.php', array('confirmupgrade' => 1, 'confirmrelease' => 1, 'cache' => 0));
+        $pluginman = core_plugin_manager::instance();
 
+        // Check for available updates.
         if ($fetchupdates) {
             // No sesskey support guaranteed here, because sessions might not work yet.
             $updateschecker = \core\update\checker::instance();
             if ($updateschecker->enabled()) {
                 $updateschecker->fetch();
             }
-            redirect($reloadurl);
+            redirect($PAGE->url);
         }
 
-        $deployer = \core\update\deployer::instance();
-        if ($deployer->enabled()) {
-            $deployer->initialize($reloadurl, $reloadurl);
+        // Cancel all plugin installations.
+        if ($abortinstallx) {
+            // No sesskey support guaranteed here, because sessions might not work yet.
+            $abortables = $pluginman->list_cancellable_installations();
+            if ($abortables) {
+                if ($confirmabortinstall) {
+                    foreach ($abortables as $plugin) {
+                        $pluginman->cancel_plugin_installation($plugin->component);
+                    }
+                    redirect($PAGE->url);
+                } else {
+                    $continue = new moodle_url($PAGE->url, array('abortinstallx' => $abortinstallx, 'confirmabortinstall' => 1));
+                    echo $output->upgrade_confirm_abort_install_page($abortables, $continue);
+                    die();
+                }
+            }
+            redirect($PAGE->url);
+        }
 
-            $deploydata = $deployer->submitted_data();
-            if (!empty($deploydata)) {
-                // No sesskey support guaranteed here, because sessions might not work yet.
-                echo $output->upgrade_plugin_confirm_deploy_page($deployer, $deploydata);
-                die();
+        // Cancel single plugin installation.
+        if ($abortinstall) {
+            // No sesskey support guaranteed here, because sessions might not work yet.
+            if ($confirmabortinstall) {
+                $pluginman->cancel_plugin_installation($abortinstall);
+                redirect($PAGE->url);
+            } else {
+                $continue = new moodle_url($PAGE->url, array('abortinstall' => $abortinstall, 'confirmabortinstall' => 1));
+                $abortable = $pluginman->get_plugin_info($abortinstall);
+                if ($pluginman->can_cancel_plugin_installation($abortable)) {
+                    echo $output->upgrade_confirm_abort_install_page(array($abortable), $continue);
+                    die();
+                }
+                redirect($PAGE->url);
+            }
+        }
+
+        // Cancel all plugins upgrades (that is, restore archived versions).
+        if ($abortupgradex) {
+            // No sesskey support guaranteed here, because sessions might not work yet.
+            $restorable = $pluginman->list_restorable_archives();
+            if ($restorable) {
+                upgrade_install_plugins($restorable, $confirmabortupgrade,
+                    get_string('cancelupgradehead', 'core_plugin'),
+                    new moodle_url($PAGE->url, array('abortupgradex' => 1, 'confirmabortupgrade' => 1))
+                );
+            }
+            redirect($PAGE->url);
+        }
+
+        // Cancel single plugin upgrade (that is, install the archived version).
+        if ($abortupgrade) {
+            // No sesskey support guaranteed here, because sessions might not work yet.
+            $restorable = $pluginman->list_restorable_archives();
+            if (isset($restorable[$abortupgrade])) {
+                $restorable = array($restorable[$abortupgrade]);
+                upgrade_install_plugins($restorable, $confirmabortupgrade,
+                    get_string('cancelupgradehead', 'core_plugin'),
+                    new moodle_url($PAGE->url, array('abortupgrade' => $abortupgrade, 'confirmabortupgrade' => 1))
+                );
+            }
+            redirect($PAGE->url);
+        }
+
+        // Install all available missing dependencies.
+        if ($installdepx) {
+            // No sesskey support guaranteed here, because sessions might not work yet.
+            $installable = $pluginman->filter_installable($pluginman->missing_dependencies(true));
+            upgrade_install_plugins($installable, $confirminstalldep,
+                get_string('dependencyinstallhead', 'core_plugin'),
+                new moodle_url($PAGE->url, array('installdepx' => 1, 'confirminstalldep' => 1))
+            );
+        }
+
+        // Install single available missing dependency.
+        if ($installdep) {
+            // No sesskey support guaranteed here, because sessions might not work yet.
+            $installable = $pluginman->filter_installable($pluginman->missing_dependencies(true));
+            if (!empty($installable[$installdep])) {
+                $installable = array($installable[$installdep]);
+                upgrade_install_plugins($installable, $confirminstalldep,
+                    get_string('dependencyinstallhead', 'core_plugin'),
+                    new moodle_url($PAGE->url, array('installdep' => $installdep, 'confirminstalldep' => 1))
+                );
+            }
+        }
+
+        // Install all available updates.
+        if ($installupdatex) {
+            // No sesskey support guaranteed here, because sessions might not work yet.
+            $installable = $pluginman->filter_installable($pluginman->available_updates());
+            upgrade_install_plugins($installable, $confirminstallupdate,
+                get_string('updateavailableinstallallhead', 'core_admin'),
+                new moodle_url($PAGE->url, array('installupdatex' => 1, 'confirminstallupdate' => 1))
+            );
+        }
+
+        // Install single available update.
+        if ($installupdate and $installupdateversion) {
+            // No sesskey support guaranteed here, because sessions might not work yet.
+            if ($pluginman->is_remote_plugin_installable($installupdate, $installupdateversion)) {
+                $installable = array($pluginman->get_remote_plugin_info($installupdate, $installupdateversion, true));
+                upgrade_install_plugins($installable, $confirminstallupdate,
+                    get_string('updateavailableinstallallhead', 'core_admin'),
+                    new moodle_url($PAGE->url, array('installupdate' => $installupdate,
+                        'installupdateversion' => $installupdateversion, 'confirminstallupdate' => 1)
+                    )
+                );
             }
         }
 
         echo $output->upgrade_plugin_check_page(core_plugin_manager::instance(), \core\update\checker::instance(),
-                $version, $showallplugins, $reloadurl,
-                new moodle_url('/admin/index.php', array('confirmupgrade'=>1, 'confirmrelease'=>1, 'confirmplugincheck'=>1, 'cache'=>0)));
+                $version, $showallplugins, $PAGE->url, new moodle_url($PAGE->url, array('confirmplugincheck' => 1)));
         die();
 
     } else {
         // Always verify plugin dependencies!
         $failed = array();
         if (!core_plugin_manager::instance()->all_plugins_ok($version, $failed)) {
-            $reloadurl = new moodle_url('/admin/index.php', array('confirmupgrade' => 1, 'confirmrelease' => 1, 'cache' => 0));
-            echo $output->unsatisfied_dependencies_page($version, $failed, $reloadurl);
+            echo $output->unsatisfied_dependencies_page($version, $failed, $PAGE->url);
             die();
         }
         unset($failed);
@@ -375,9 +515,17 @@ if (!$cache and $branch <> $CFG->branch) {  // Update the branch
 }
 
 if (!$cache and moodle_needs_upgrading()) {
+
+    $PAGE->set_url(new moodle_url($PAGE->url, array('confirmplugincheck' => $confirmplugins)));
+
+    check_upgrade_key($upgradekeyhash);
+
     if (!$PAGE->headerprinted) {
         // means core upgrade or installation was not already done
 
+        $pluginman = core_plugin_manager::instance();
+        $output = $PAGE->get_renderer('core', 'admin');
+
         if (!$confirmplugins) {
             $strplugincheck = get_string('plugincheck');
 
@@ -386,6 +534,7 @@ if (!$cache and moodle_needs_upgrading()) {
             $PAGE->set_heading($strplugincheck);
             $PAGE->set_cacheable(false);
 
+            // Check for available updates.
             if ($fetchupdates) {
                 require_sesskey();
                 $updateschecker = \core\update\checker::instance();
@@ -395,36 +544,130 @@ if (!$cache and moodle_needs_upgrading()) {
                 redirect($PAGE->url);
             }
 
-            /** @var core_admin_renderer $output */
-            $output = $PAGE->get_renderer('core', 'admin');
+            // Cancel all plugin installations.
+            if ($abortinstallx) {
+                require_sesskey();
+                $abortables = $pluginman->list_cancellable_installations();
+                if ($abortables) {
+                    if ($confirmabortinstall) {
+                        foreach ($abortables as $plugin) {
+                            $pluginman->cancel_plugin_installation($plugin->component);
+                        }
+                        redirect($PAGE->url);
+                    } else {
+                        $continue = new moodle_url($PAGE->url, array('abortinstallx' => $abortinstallx,
+                            'confirmabortinstall' => 1));
+                        echo $output->upgrade_confirm_abort_install_page($abortables, $continue);
+                        die();
+                    }
+                }
+                redirect($PAGE->url);
+            }
 
-            $deployer = \core\update\deployer::instance();
-            if ($deployer->enabled()) {
-                $deployer->initialize($PAGE->url, $PAGE->url);
+            // Cancel single plugin installation.
+            if ($abortinstall) {
+                require_sesskey();
+                if ($confirmabortinstall) {
+                    $pluginman->cancel_plugin_installation($abortinstall);
+                    redirect($PAGE->url);
+                } else {
+                    $continue = new moodle_url($PAGE->url, array('abortinstall' => $abortinstall, 'confirmabortinstall' => 1));
+                    $abortable = $pluginman->get_plugin_info($abortinstall);
+                    if ($pluginman->can_cancel_plugin_installation($abortable)) {
+                        echo $output->upgrade_confirm_abort_install_page(array($abortable), $continue);
+                        die();
+                    }
+                    redirect($PAGE->url);
+                }
+            }
 
-                $deploydata = $deployer->submitted_data();
-                if (!empty($deploydata)) {
-                    require_sesskey();
-                    echo $output->upgrade_plugin_confirm_deploy_page($deployer, $deploydata);
-                    die();
+            // Cancel all plugins upgrades (that is, restore archived versions).
+            if ($abortupgradex) {
+                require_sesskey();
+                $restorable = $pluginman->list_restorable_archives();
+                if ($restorable) {
+                    upgrade_install_plugins($restorable, $confirmabortupgrade,
+                        get_string('cancelupgradehead', 'core_plugin'),
+                        new moodle_url($PAGE->url, array('abortupgradex' => 1, 'confirmabortupgrade' => 1))
+                    );
+                }
+                redirect($PAGE->url);
+            }
+
+            // Cancel single plugin upgrade (that is, install the archived version).
+            if ($abortupgrade) {
+                require_sesskey();
+                $restorable = $pluginman->list_restorable_archives();
+                if (isset($restorable[$abortupgrade])) {
+                    $restorable = array($restorable[$abortupgrade]);
+                    upgrade_install_plugins($restorable, $confirmabortupgrade,
+                        get_string('cancelupgradehead', 'core_plugin'),
+                        new moodle_url($PAGE->url, array('abortupgrade' => $abortupgrade, 'confirmabortupgrade' => 1))
+                    );
+                }
+                redirect($PAGE->url);
+            }
+
+            // Install all available missing dependencies.
+            if ($installdepx) {
+                require_sesskey();
+                $installable = $pluginman->filter_installable($pluginman->missing_dependencies(true));
+                upgrade_install_plugins($installable, $confirminstalldep,
+                    get_string('dependencyinstallhead', 'core_plugin'),
+                    new moodle_url($PAGE->url, array('installdepx' => 1, 'confirminstalldep' => 1))
+                );
+            }
+
+            // Install single available missing dependency.
+            if ($installdep) {
+                require_sesskey();
+                $installable = $pluginman->filter_installable($pluginman->missing_dependencies(true));
+                if (!empty($installable[$installdep])) {
+                    $installable = array($installable[$installdep]);
+                    upgrade_install_plugins($installable, $confirminstalldep,
+                        get_string('dependencyinstallhead', 'core_plugin'),
+                        new moodle_url($PAGE->url, array('installdep' => $installdep, 'confirminstalldep' => 1))
+                    );
+                }
+            }
+
+            // Install all available updates.
+            if ($installupdatex) {
+                require_sesskey();
+                $installable = $pluginman->filter_installable($pluginman->available_updates());
+                upgrade_install_plugins($installable, $confirminstallupdate,
+                    get_string('updateavailableinstallallhead', 'core_admin'),
+                    new moodle_url($PAGE->url, array('installupdatex' => 1, 'confirminstallupdate' => 1))
+                );
+            }
+
+            // Install single available update.
+            if ($installupdate and $installupdateversion) {
+                require_sesskey();
+                if ($pluginman->is_remote_plugin_installable($installupdate, $installupdateversion)) {
+                    $installable = array($pluginman->get_remote_plugin_info($installupdate, $installupdateversion, true));
+                    upgrade_install_plugins($installable, $confirminstallupdate,
+                        get_string('updateavailableinstallallhead', 'core_admin'),
+                        new moodle_url($PAGE->url, array('installupdate' => $installupdate,
+                            'installupdateversion' => $installupdateversion, 'confirminstallupdate' => 1)
+                        )
+                    );
                 }
             }
 
             // Show plugins info.
-            echo $output->upgrade_plugin_check_page(core_plugin_manager::instance(), \core\update\checker::instance(),
+            echo $output->upgrade_plugin_check_page($pluginman, \core\update\checker::instance(),
                     $version, $showallplugins,
                     new moodle_url($PAGE->url),
-                    new moodle_url('/admin/index.php', array('confirmplugincheck'=>1, 'cache'=>0)));
+                    new moodle_url($PAGE->url, array('confirmplugincheck' => 1, 'cache' => 0)));
             die();
         }
 
         // Make sure plugin dependencies are always checked.
         $failed = array();
-        if (!core_plugin_manager::instance()->all_plugins_ok($version, $failed)) {
-            /** @var core_admin_renderer $output */
+        if (!$pluginman->all_plugins_ok($version, $failed)) {
             $output = $PAGE->get_renderer('core', 'admin');
-            $reloadurl = new moodle_url('/admin/index.php', array('cache' => 0));
-            echo $output->unsatisfied_dependencies_page($version, $failed, $reloadurl);
+            echo $output->unsatisfied_dependencies_page($version, $failed, $PAGE->url);
             die();
         }
         unset($failed);
@@ -514,7 +757,15 @@ if (isguestuser()) {
     redirect(get_login_url());
 }
 $context = context_system::instance();
-require_capability('moodle/site:config', $context);
+
+if (!has_capability('moodle/site:config', $context)) {
+    // Do not throw exception display an empty page with administration menu if visible for current user.
+    $PAGE->set_title($SITE->fullname);
+    $PAGE->set_heading($SITE->fullname);
+    echo $OUTPUT->header();
+    echo $OUTPUT->footer();
+    exit;
+}
 
 // check that site is properly customized
 $site = get_site();
@@ -522,14 +773,10 @@ if (empty($site->shortname)) {
     // probably new installation - lets return to frontpage after this step
     // remove settings that we want uninitialised
     unset_config('registerauth');
+    unset_config('timezone'); // Force admin to select timezone!
     redirect('upgradesettings.php?return=site');
 }
 
-// Check if we are returning from moodle.org registration and if so, we mark that fact to remove reminders
-if (!empty($id) and $id == $CFG->siteidentifier) {
-    set_config('registered', time());
-}
-
 // setup critical warnings before printing admin tree block
 $insecuredataroot = is_dataroot_insecure(true);
 $SESSION->admin_critical_warning = ($insecuredataroot==INSECURE_DATAROOT_ERROR);
@@ -566,7 +813,7 @@ $updateschecker = \core\update\checker::instance();
 $availableupdates = array();
 $availableupdatesfetch = null;
 
-if (empty($CFG->disableupdatenotifications)) {
+if ($updateschecker->enabled()) {
     // Only compute the update information when it is going to be displayed to the user.
     $availableupdates['core'] = $updateschecker->get_update_info('core',
         array('minmaturity' => $CFG->updateminmaturity, 'notifybuilds' => $CFG->updatenotifybuilds));
@@ -575,14 +822,13 @@ if (empty($CFG->disableupdatenotifications)) {
     $pluginman = core_plugin_manager::instance();
     foreach ($pluginman->get_plugins() as $plugintype => $plugintypeinstances) {
         foreach ($plugintypeinstances as $pluginname => $plugininfo) {
-            if (!empty($plugininfo->availableupdates)) {
-                foreach ($plugininfo->availableupdates as $pluginavailableupdate) {
-                    if ($pluginavailableupdate->version > $plugininfo->versiondisk) {
-                        if (!isset($availableupdates[$plugintype.'_'.$pluginname])) {
-                            $availableupdates[$plugintype.'_'.$pluginname] = array();
-                        }
-                        $availableupdates[$plugintype.'_'.$pluginname][] = $pluginavailableupdate;
+            $pluginavailableupdates = $plugininfo->available_updates();
+            if (!empty($pluginavailableupdates)) {
+                foreach ($pluginavailableupdates as $pluginavailableupdate) {
+                    if (!isset($availableupdates[$plugintype.'_'.$pluginname])) {
+                        $availableupdates[$plugintype.'_'.$pluginname] = array();
                     }
+                    $availableupdates[$plugintype.'_'.$pluginname][] = $pluginavailableupdate;
                 }
             }
         }
@@ -600,7 +846,6 @@ $cachewarnings = cache_helper::warnings();
 
 admin_externalpage_setup('adminnotifications');
 
-/* @var core_admin_renderer $output */
 $output = $PAGE->get_renderer('core', 'admin');
 
 echo $output->admin_notifications_page($maturity, $insecuredataroot, $errorsdisplayed, $cronoverdue, $dbproblems,
