@@ -23,5 +23,5 @@
  */
 
 $string['calendar_month:addinstance'] = 'Add a new calendar block';
-$string['calendar_month:myaddinstance'] = 'Add a new calendar block to My home';
+$string['calendar_month:myaddinstance'] = 'Add a new calendar block to Dashboard';
 $string['pluginname'] = 'Calendar';
