@@ -27,4 +27,4 @@
 $string['enabledock'] = 'Allow the user to dock this block';
 $string['pluginname'] = 'Administration';
 $string['settings:addinstance'] = 'Add a new administration block';
-$string['settings:myaddinstance'] = 'Add a new administration block to My home';
+$string['settings:myaddinstance'] = 'Add a new administration block to Dashboard';
