@@ -34,7 +34,8 @@ var MENUTEMPLATE = '' +
             'title="{{title}}">' +
             '<img class="icon" aria-hidden="true" role="presentation" width="16" height="16" '+
                 'style="background-color:{{config.menuColor}};" src="{{config.iconurl}}" />' +
-            '<img class="icon" aria-hidden="true" role="presentation" width="16" height="16" src="{{image_url "t/expanded" "moodle"}}"/>' +
+            '<img class="icon" aria-hidden="true" role="presentation" width="16" height="16" ' +
+                'src="{{image_url "t/expanded" "moodle"}}"/>' +
         '</button>';
 
 var DISABLED = 'disabled',
@@ -176,6 +177,7 @@ EditorPluginButtons.prototype = {
      * specified, in the class for the button.
      * @param {function} config.callback A callback function to call when the button is clicked.
      * @param {object} [config.callbackArgs] Any arguments to pass to the callback.
+     * @param {boolean} [config.inlineFormat] Delay callback for text input if selection is collapsed.
      * @return {Node} The Node representing the newly created button.
      */
     addButton: function(config) {
@@ -208,7 +210,8 @@ EditorPluginButtons.prototype = {
         // Create the actual button.
         button = Y.Node.create('<button type="button" class="' + buttonClass + '"' +
                 'tabindex="-1">' +
-                    '<img class="icon" aria-hidden="true" role="presentation" width="16" height="16" src="' + config.iconurl + '"/>' +
+                    '<img class="icon" aria-hidden="true" role="presentation" width="16" height="16" src="' +
+                            config.iconurl + '"/>' +
                 '</button>');
         button.setAttribute('title', title);
 
@@ -281,7 +284,8 @@ EditorPluginButtons.prototype = {
     /**
      * Add a basic button which ties into the execCommand.
      *
-     * See {{#crossLink "M.editor_atto.EditorPluginButtons/addButton:method"}}addButton{{/crossLink}} for full details of the optional parameters.
+     * See {{#crossLink "M.editor_atto.EditorPluginButtons/addButton:method"}}addButton{{/crossLink}}
+     * for full details of the optional parameters.
      *
      * @method addBasicButton
      * @param {object} config The button configuration
@@ -337,6 +341,7 @@ EditorPluginButtons.prototype = {
      * specified, in the class for the button.
      * @param {function} config.callback A callback function to call when the button is clicked.
      * @param {object} [config.callbackArgs] Any arguments to pass to the callback.
+     * @param {boolean} [config.inlineFormat] Delay callback for text input if selection is collapsed.
      * @param {array} config.entries List of menu entries with the string (entry.text) and the handlers (entry.handler).
      * @param {number} [config.overlayWidth=14] The width of the menu. This will be suffixed with the 'em' unit.
      * @param {string} [config.menuColor] menu icon background color
@@ -510,7 +515,6 @@ EditorPluginButtons.prototype = {
         // Cancel the hide menu prevention.
         menuDialogue.set('preventHideMenu', false);
 
-        console.log('Menu item chosen');
         // Set the focus after hide so that focus is returned to the editor and changes are made correctly.
         menuDialogue.set('focusAfterHide', this.get('host').editor);
         menuDialogue.hide(e);
@@ -523,6 +527,7 @@ EditorPluginButtons.prototype = {
      * @param {object} config
      * @param {function} config.callback A callback function to call when the button is clicked.
      * @param {object} [config.callbackArgs] Any arguments to pass to the callback.
+     * @param {boolean} [config.inlineFormat] Delay callback for text input if selection is collapsed.
      * @param {object} [inheritFrom] A parent configuration that this configuration may inherit from.
      * @return {object} The normalized configuration
      * @private
@@ -538,9 +543,18 @@ EditorPluginButtons.prototype = {
             inheritFrom = {};
         }
 
+
+        // First we wrap the callback in function to handle formating of text inserted into collapsed selection.
+        config.inlineFormat = config.inlineFormat || inheritFrom.inlineFormat;
+        config._inlineCallback = config.callback || inheritFrom.callback;
+        config._callback = config.callback || inheritFrom.callback;
+        if (config.inlineFormat && typeof config._inlineCallback === 'function') {
+            config._callback = function(e, args) {
+                this.get('host').applyFormat(e, config._inlineCallback, this, args);
+            };
+        }
         // We wrap the callback in function to prevent the default action, check whether the editor is
         // active and focus it, and then mark the field as updated.
-        config._callback = config.callback || inheritFrom.callback;
         config.callback = Y.rbind(this._callbackWrapper, this, config._callback, config.callbackArgs);
 
         config._callbackNormalized = true;
