@@ -48,7 +48,7 @@ function external_function_info($function, $strictness=MUST_EXIST) {
         // Fallback to explicit include of externallib.php.
         $function->classpath = empty($function->classpath) ? core_component::get_component_directory($function->component).'/externallib.php' : $CFG->dirroot.'/'.$function->classpath;
         if (!file_exists($function->classpath)) {
-            throw new coding_exception('Cannot find file with external function implementation');
+            throw new coding_exception('Cannot find file with external function implementation: ' . $function->classname);
         }
         require_once($function->classpath);
         if (!class_exists($function->classname)) {
@@ -56,8 +56,10 @@ function external_function_info($function, $strictness=MUST_EXIST) {
         }
     }
 
+    $function->ajax_method = $function->methodname.'_is_allowed_from_ajax';
     $function->parameters_method = $function->methodname.'_parameters';
     $function->returns_method    = $function->methodname.'_returns';
+    $function->deprecated_method = $function->methodname.'_is_deprecated';
 
     // make sure the implementaion class is ok
     if (!method_exists($function->classname, $function->methodname)) {
@@ -69,6 +71,12 @@ function external_function_info($function, $strictness=MUST_EXIST) {
     if (!method_exists($function->classname, $function->returns_method)) {
         throw new coding_exception('Missing returned values description');
     }
+    if (method_exists($function->classname, $function->deprecated_method)) {
+        if (call_user_func(array($function->classname, $function->deprecated_method)) === true) {
+            $function->deprecated = true;
+        }
+    }
+    $function->allowed_from_ajax = false;
 
     // fetch the parameters description
     $function->parameters_desc = call_user_func(array($function->classname, $function->parameters_method));
@@ -99,6 +107,23 @@ function external_function_info($function, $strictness=MUST_EXIST) {
         if (isset($functions[$function->name]['testclientpath'])) {
             $function->testclientpath = $functions[$function->name]['testclientpath'];
         }
+        if (isset($functions[$function->name]['type'])) {
+            $function->type = $functions[$function->name]['type'];
+        }
+        if (isset($functions[$function->name]['ajax'])) {
+            $function->allowed_from_ajax = $functions[$function->name]['ajax'];
+        } else if (method_exists($function->classname, $function->ajax_method)) {
+            if (call_user_func(array($function->classname, $function->ajax_method)) === true) {
+                debugging('External function ' . $function->ajax_method . '() function is deprecated.' .
+                          'Set ajax=>true in db/service.php instead.', DEBUG_DEVELOPER);
+                $function->allowed_from_ajax = true;
+            }
+        }
+        if (isset($functions[$function->name]['loginrequired'])) {
+            $function->loginrequired = $functions[$function->name]['loginrequired'];
+        } else {
+            $function->loginrequired = true;
+        }
     }
 
     return $function;
@@ -333,7 +358,7 @@ class external_api {
      * @param stdClass $context
      * @since Moodle 2.0
      */
-    protected static function validate_context($context) {
+    public static function validate_context($context) {
         global $CFG;
 
         if (empty($context)) {
@@ -526,6 +551,30 @@ class external_multiple_structure extends external_description {
  * @since Moodle 2.0
  */
 class external_function_parameters extends external_single_structure {
+
+    /**
+     * Constructor - does extra checking to prevent top level optional parameters.
+     *
+     * @param array $keys
+     * @param string $desc
+     * @param bool $required
+     * @param array $default
+     */
+    public function __construct(array $keys, $desc='', $required=VALUE_REQUIRED, $default=null) {
+        global $CFG;
+
+        if ($CFG->debugdeveloper) {
+            foreach ($keys as $key => $value) {
+                if ($value instanceof external_value) {
+                    if ($value->required == VALUE_OPTIONAL) {
+                        debugging('External function parameters: invalid OPTIONAL value specified.', DEBUG_DEVELOPER);
+                        break;
+                    }
+                }
+            }
+        }
+        parent::__construct($keys, $desc, $required, $default);
+    }
 }
 
 /**
@@ -705,6 +754,38 @@ function external_validate_format($format) {
     return $format;
 }
 
+/**
+ * Format the string to be returned properly as requested by the either the web service server,
+ * either by an internally call.
+ * The caller can change the format (raw) with the external_settings singleton
+ * All web service servers must set this singleton when parsing the $_GET and $_POST.
+ *
+ * @param string $str The string to be filtered. Should be plain text, expect
+ * possibly for multilang tags.
+ * @param boolean $striplinks To strip any link in the result text. Moodle 1.8 default changed from false to true! MDL-8713
+ * @param int $contextid The id of the context for the string (affects filters).
+ * @param array $options options array/object or courseid
+ * @return string text
+ * @since Moodle 3.0
+ */
+function external_format_string($str, $contextid, $striplinks = true, $options = array()) {
+
+    // Get settings (singleton).
+    $settings = external_settings::get_instance();
+    if (empty($contextid)) {
+        throw new coding_exception('contextid is required');
+    }
+
+    if (!$settings->get_raw()) {
+        $context = context::instance_by_id($contextid);
+        $options['context'] = $context;
+        $options['filter'] = $settings->get_filter();
+        $str = format_string($str, $striplinks, $options);
+    }
+
+    return $str;
+}
+
 /**
  * Format the text to be returned properly as requested by the either the web service server,
  * either by an internally call.
@@ -732,7 +813,8 @@ function external_format_text($text, $textformat, $contextid, $component, $filea
     }
 
     if (!$settings->get_raw()) {
-        $text = format_text($text, $textformat, array('para' => false, 'filter' => $settings->get_filter()));
+        $context = context::instance_by_id($contextid);
+        $text = format_text($text, $textformat, array('para' => false, 'filter' => $settings->get_filter(), 'context' => $context));
         $textformat = FORMAT_HTML; // Once converted to html (from markdown, plain... lets inform consumer this is already HTML).
     }
 
@@ -770,6 +852,10 @@ class external_settings {
      * Constructor - protected - can not be instanciated
      */
     protected function __construct() {
+        if (!defined('AJAX_SCRIPT') && !defined('CLI_SCRIPT') && !defined('WS_SERVER')) {
+            // For normal pages, the default should match the default for format_text.
+            $this->filter = true;
+        }
     }
 
     /**
@@ -863,3 +949,46 @@ class external_settings {
         return $this->file;
     }
 }
+
+/**
+ * Utility functions for the external API.
+ *
+ * @package    core_webservice
+ * @copyright  2015 Juan Leyva
+ * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
+ * @since Moodle 3.0
+ */
+class external_util {
+
+    /**
+     * Validate a list of courses, returning the complete course objects for valid courses.
+     *
+     * @param  array $courseids A list of course ids
+     * @return array            An array of courses and the validation warnings
+     */
+    public static function validate_courses($courseids) {
+        // Delete duplicates.
+        $courseids = array_unique($courseids);
+        $courses = array();
+        $warnings = array();
+
+        foreach ($courseids as $cid) {
+            // Check the user can function in this context.
+            try {
+                $context = context_course::instance($cid);
+                external_api::validate_context($context);
+                $courses[$cid] = get_course($cid);
+            } catch (Exception $e) {
+                $warnings[] = array(
+                    'item' => 'course',
+                    'itemid' => $cid,
+                    'warningcode' => '1',
+                    'message' => 'No access rights in course context'
+                );
+            }
+        }
+
+        return array($courses, $warnings);
+    }
+
+}
