@@ -47,9 +47,12 @@ require_once(dirname(dirname(__FILE__)) . '/message/lib.php');
  *  contexturl string if this is a notification then you can specify a url to view the event. For example the forum post the user is being notified of.
  *  contexturlname string the display text for contexturl
  *
+ * Note: processor failure is is not reported as false return value,
+ *       earlier versions did not do it consistently either.
+ *
  * @category message
- * @param object $eventdata information about the message (component, userfrom, userto, ...)
- * @return mixed the integer ID of the new message or false if there was a problem with a processor
+ * @param stdClass|\core\message\message $eventdata information about the message (component, userfrom, userto, ...)
+ * @return mixed the integer ID of the new message or false if there was a problem with submitted data
  */
 function message_send($eventdata) {
     global $CFG, $DB;
@@ -65,20 +68,33 @@ function message_send($eventdata) {
         return $messageid;
     }
 
-    //TODO: we need to solve problems with database transactions here somehow, for now we just prevent transactions - sorry
-    $DB->transactions_forbidden();
-
     // By default a message is a notification. Only personal/private messages aren't notifications.
     if (!isset($eventdata->notification)) {
         $eventdata->notification = 1;
     }
 
-    if (is_number($eventdata->userto)) {
+    if (!is_object($eventdata->userto)) {
         $eventdata->userto = core_user::get_user($eventdata->userto);
     }
-    if (is_int($eventdata->userfrom)) {
+    if (!is_object($eventdata->userfrom)) {
         $eventdata->userfrom = core_user::get_user($eventdata->userfrom);
     }
+    if (!$eventdata->userto) {
+        debugging('Attempt to send msg to unknown user', DEBUG_NORMAL);
+        return false;
+    }
+    if (!$eventdata->userfrom) {
+        debugging('Attempt to send msg from unknown user', DEBUG_NORMAL);
+        return false;
+    }
+
+    // Verify all necessary data fields are present.
+    if (!isset($eventdata->userto->auth) or !isset($eventdata->userto->suspended)
+            or !isset($eventdata->userto->deleted) or !isset($eventdata->userto->emailstop)) {
+
+        debugging('Necessary properties missing in userto object, fetching full record', DEBUG_DEVELOPER);
+        $eventdata->userto = core_user::get_user($eventdata->userto->id);
+    }
 
     $usertoisrealuser = (core_user::is_real_user($eventdata->userto->id) != false);
     // If recipient is internal user (noreply user), and emailstop is set then don't send any msg.
@@ -87,10 +103,6 @@ function message_send($eventdata) {
         return false;
     }
 
-    if (!isset($eventdata->userto->auth) or !isset($eventdata->userto->suspended) or !isset($eventdata->userto->deleted)) {
-        $eventdata->userto = core_user::get_user($eventdata->userto->id);
-    }
-
     //after how long inactive should the user be considered logged off?
     if (isset($CFG->block_online_users_timetosee)) {
         $timetoshowusers = $CFG->block_online_users_timetosee * 60;
@@ -117,13 +129,13 @@ function message_send($eventdata) {
     $savemessage->notification      = $eventdata->notification;
 
     if (!empty($eventdata->contexturl)) {
-        $savemessage->contexturl = $eventdata->contexturl;
+        $savemessage->contexturl = (string)$eventdata->contexturl;
     } else {
         $savemessage->contexturl = null;
     }
 
     if (!empty($eventdata->contexturlname)) {
-        $savemessage->contexturlname = $eventdata->contexturlname;
+        $savemessage->contexturlname = (string)$eventdata->contexturlname;
     } else {
         $savemessage->contexturlname = null;
     }
@@ -189,12 +201,6 @@ function message_send($eventdata) {
             debugging('Attempt to force message delivery to user who has "'.$processor->name.'" output unconfigured', DEBUG_NORMAL);
         }
 
-        // Warn developers that necessary data is missing regardless of how the processors are configured
-        if (!isset($eventdata->userto->emailstop)) {
-            debugging('userto->emailstop is not set. Retrieving it from the user table');
-            $eventdata->userto->emailstop = $DB->get_field('user', 'emailstop', array('id'=>$eventdata->userto->id));
-        }
-
         // Populate the list of processors we will be using
         if ($permitted == 'forced' && $userisconfigured) {
             // An admin is forcing users to use this message processor. Use this processor unconditionally.
@@ -203,7 +209,7 @@ function message_send($eventdata) {
             // User has not disabled notifications
             // See if user set any notification preferences, otherwise use site default ones
             $userpreferencename = 'message_provider_'.$preferencebase.'_'.$userstate;
-            if ($userpreference = get_user_preferences($userpreferencename, null, $eventdata->userto->id)) {
+            if ($userpreference = get_user_preferences($userpreferencename, null, $eventdata->userto)) {
                 if (in_array($processor->name, explode(',', $userpreference))) {
                     $processorlist[] = $processor->name;
                 }
@@ -215,60 +221,12 @@ function message_send($eventdata) {
         }
     }
 
-    if (empty($processorlist) && $savemessage->notification) {
-        //if they have deselected all processors and its a notification mark it read. The user doesnt want to be bothered
-        $savemessage->timeread = time();
-        $messageid = $DB->insert_record('message_read', $savemessage);
-    } else {                        // Process the message
-        // Store unread message just in case we can not send it
-        $messageid = $savemessage->id = $DB->insert_record('message', $savemessage);
-        $eventdata->savedmessageid = $savemessage->id;
+    // Store unread message just in case we get a fatal error any time later.
+    $savemessage->id = $DB->insert_record('message', $savemessage);
+    $eventdata->savedmessageid = $savemessage->id;
 
-        // Try to deliver the message to each processor
-        if (!empty($processorlist)) {
-            foreach ($processorlist as $procname) {
-                if (!$processors[$procname]->object->send_message($eventdata)) {
-                    debugging('Error calling message processor '.$procname);
-                    $messageid = false;
-                }
-            }
-
-            //if messaging is disabled and they previously had forum notifications handled by the popup processor
-            //or any processor that puts a row in message_working then the notification will remain forever
-            //unread. To prevent this mark the message read if messaging is disabled
-            if (empty($CFG->messaging)) {
-                require_once($CFG->dirroot.'/message/lib.php');
-                $messageid = message_mark_message_read($savemessage, time());
-            } else if ( $DB->count_records('message_working', array('unreadmessageid' => $savemessage->id)) == 0){
-                //if there is no more processors that want to process this we can move message to message_read
-                require_once($CFG->dirroot.'/message/lib.php');
-                $messageid = message_mark_message_read($savemessage, time(), true);
-            }
-        }
-    }
-
-    // We may be sending a message from the 'noreply' address, which means we are not actually sending a
-    // message from a valid user. In this case, we will set the userid to 0.
-    // Check if the userid is valid.
-    if (core_user::is_real_user($eventdata->userfrom->id)) {
-        $userfromid = $eventdata->userfrom->id;
-    } else {
-        $userfromid = 0;
-    }
-
-    // Trigger event for sending a message.
-    $event = \core\event\message_sent::create(array(
-        'userid' => $userfromid,
-        'context'  => context_system::instance(),
-        'relateduserid' => $eventdata->userto->id,
-        'other' => array(
-            'messageid' => $messageid // Can't use this as the objectid as it can either be the id in the 'message_read'
-                                      // or 'message' table.
-        )
-    ));
-    $event->trigger();
-
-    return $messageid;
+    // Let the manager do the sending or buffering when db transaction in progress.
+    return \core\message\manager::send_message($eventdata, $savemessage, $processorlist);
 }
 
 
