@@ -32,7 +32,7 @@
  * @copyright  2012 Petr Skoda {@link http://skodak.org}
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-abstract class advanced_testcase extends PHPUnit_Framework_TestCase {
+abstract class advanced_testcase extends base_testcase {
     /** @var bool automatically reset everything? null means log changes */
     private $resetAfterTest;
 
@@ -83,14 +83,21 @@ abstract class advanced_testcase extends PHPUnit_Framework_TestCase {
 
             // Deal with any debugging messages.
             $debugerror = phpunit_util::display_debugging_messages();
-            phpunit_util::reset_debugging();
+            $this->resetDebugging();
             if ($debugerror) {
-                trigger_error('Unenxpected debugging() call detected.', E_USER_NOTICE);
+                trigger_error('Unexpected debugging() call detected.', E_USER_NOTICE);
             }
 
-        } catch (Exception $e) {
+        } catch (Exception $ex) {
+            $e = $ex;
+        } catch (Throwable $ex) {
+            // Engine errors in PHP7 throw exceptions of type Throwable (this "catch" will be ignored in PHP5).
+            $e = $ex;
+        }
+
+        if (isset($e)) {
             // cleanup after failed expectation
-            phpunit_util::reset_all_data();
+            self::resetAllData();
             throw $e;
         }
 
@@ -104,7 +111,7 @@ abstract class advanced_testcase extends PHPUnit_Framework_TestCase {
                 phpunit_util::reset_all_database_sequences();
                 phpunit_util::$lastdbwrites = $DB->perf_get_writes(); // no db reset necessary
             }
-            phpunit_util::reset_all_data(null);
+            self::resetAllData(null);
 
         } else if ($this->resetAfterTest === false) {
             if ($this->testdbtransaction) {
@@ -118,16 +125,16 @@ abstract class advanced_testcase extends PHPUnit_Framework_TestCase {
                 try {
                     $this->testdbtransaction->allow_commit();
                 } catch (dml_transaction_exception $e) {
-                    phpunit_util::reset_all_data();
+                    self::resetAllData();
                     throw new coding_exception('Invalid transaction state detected in test '.$this->getName());
                 }
             }
-            phpunit_util::reset_all_data(true);
+            self::resetAllData(true);
         }
 
         // make sure test did not forget to close transaction
         if ($DB->is_transaction_started()) {
-            phpunit_util::reset_all_data();
+            self::resetAllData();
             if ($this->getStatus() == PHPUnit_Runner_BaseTestRunner::STATUS_PASSED
                 or $this->getStatus() == PHPUnit_Runner_BaseTestRunner::STATUS_SKIPPED
                 or $this->getStatus() == PHPUnit_Runner_BaseTestRunner::STATUS_INCOMPLETE) {
@@ -272,7 +279,7 @@ abstract class advanced_testcase extends PHPUnit_Framework_TestCase {
      * @param string $message
      */
     public function assertDebuggingCalled($debugmessage = null, $debuglevel = null, $message = '') {
-        $debugging = phpunit_util::get_debugging_messages();
+        $debugging = $this->getDebuggingMessages();
         $count = count($debugging);
 
         if ($count == 0) {
@@ -297,7 +304,7 @@ abstract class advanced_testcase extends PHPUnit_Framework_TestCase {
             $this->assertSame($debuglevel, $debug->level, $message);
         }
 
-        phpunit_util::reset_debugging();
+        $this->resetDebugging();
     }
 
     /**
@@ -305,7 +312,7 @@ abstract class advanced_testcase extends PHPUnit_Framework_TestCase {
      * @param string $message
      */
     public function assertDebuggingNotCalled($message = '') {
-        $debugging = phpunit_util::get_debugging_messages();
+        $debugging = $this->getDebuggingMessages();
         $count = count($debugging);
 
         if ($message === '') {
@@ -446,16 +453,21 @@ abstract class advanced_testcase extends PHPUnit_Framework_TestCase {
      * @return void
      */
     public static function tearDownAfterClass() {
-        phpunit_util::reset_all_data();
+        self::resetAllData();
     }
 
+
     /**
      * Reset all database tables, restore global state and clear caches and optionally purge dataroot dir.
-     * @static
+     *
+     * @param bool $detectchanges
+     *      true  - changes in global state and database are reported as errors
+     *      false - no errors reported
+     *      null  - only critical problems are reported as errors
      * @return void
      */
-    public static function resetAllData() {
-        phpunit_util::reset_all_data();
+    public static function resetAllData($detectchanges = false) {
+        phpunit_util::reset_all_data($detectchanges);
     }
 
     /**
@@ -480,6 +492,9 @@ abstract class advanced_testcase extends PHPUnit_Framework_TestCase {
         unset($user->access);
         unset($user->preference);
 
+        // Enusre session is empty, as it may contain caches and user specific info.
+        \core\session\manager::init_empty_session();
+
         \core\session\manager::set_user($user);
     }
 
@@ -501,6 +516,19 @@ abstract class advanced_testcase extends PHPUnit_Framework_TestCase {
         self::setUser(1);
     }
 
+    /**
+     * Change server and default php timezones.
+     *
+     * @param string $servertimezone timezone to set in $CFG->timezone (not validated)
+     * @param string $defaultphptimezone timezone to fake default php timezone (must be valid)
+     */
+    public static function setTimezone($servertimezone = 'Australia/Perth', $defaultphptimezone = 'Australia/Perth') {
+        global $CFG;
+        $CFG->timezone = $servertimezone;
+        core_date::phpunit_override_default_php_timezone($defaultphptimezone);
+        core_date::set_default_server_timezone();
+    }
+
     /**
      * Get data generator
      * @static
