@@ -59,6 +59,11 @@ abstract class testing_util {
      */
     protected static $tablestructure = null;
 
+    /**
+     * @var array keep list of sequenceid used in a table.
+     */
+    private static $tablesequences = array();
+
     /**
      * @var array original structure of all database tables
      */
@@ -74,6 +79,11 @@ abstract class testing_util {
      */
     private static $originaldatafilesjsonadded = false;
 
+    /**
+     * @var int next sequence value for a single test cycle.
+     */
+    protected static $sequencenextstartingid = null;
+
     /**
      * Return the name of the JSON file containing the init filenames.
      *
@@ -259,17 +269,15 @@ abstract class testing_util {
      * @return array  $table=>$records
      */
     protected static function get_tabledata() {
-        global $CFG;
-
-        $framework = self::get_framework();
-
-        $datafile = self::get_dataroot() . '/' . $framework . '/tabledata.ser';
-        if (!file_exists($datafile)) {
-            // Not initialised yet.
-            return array();
-        }
-
         if (!isset(self::$tabledata)) {
+            $framework = self::get_framework();
+
+            $datafile = self::get_dataroot() . '/' . $framework . '/tabledata.ser';
+            if (!file_exists($datafile)) {
+                // Not initialised yet.
+                return array();
+            }
+
             $data = file_get_contents($datafile);
             self::$tabledata = unserialize($data);
         }
@@ -287,17 +295,15 @@ abstract class testing_util {
      * @return array $table=>$records
      */
     public static function get_tablestructure() {
-        global $CFG;
-
-        $framework = self::get_framework();
-
-        $structurefile = self::get_dataroot() . '/' . $framework . '/tablestructure.ser';
-        if (!file_exists($structurefile)) {
-            // Not initialised yet.
-            return array();
-        }
-
         if (!isset(self::$tablestructure)) {
+            $framework = self::get_framework();
+
+            $structurefile = self::get_dataroot() . '/' . $framework . '/tablestructure.ser';
+            if (!file_exists($structurefile)) {
+                // Not initialised yet.
+                return array();
+            }
+
             $data = file_get_contents($structurefile);
             self::$tablestructure = unserialize($data);
         }
@@ -357,9 +363,10 @@ abstract class testing_util {
                     // incorrect table match caused by _
                     continue;
                 }
+
                 if (!is_null($info->auto_increment)) {
                     $table = preg_replace('/^'.preg_quote($prefix, '/').'/', '', $table);
-                    if ($info->auto_increment == 1) {
+                    if (isset(self::$tablesequences[$table]) && ($info->auto_increment == self::$tablesequences[$table])) {
                         $empties[$table] = $table;
                     }
                 }
@@ -409,6 +416,35 @@ abstract class testing_util {
         }
     }
 
+    /**
+     * Determine the next unique starting id sequences.
+     *
+     * @static
+     * @param array $records The records to use to determine the starting value for the table.
+     * @param string $table table name.
+     * @return int The value the sequence should be set to.
+     */
+    private static function get_next_sequence_starting_value($records, $table) {
+        if (isset(self::$tablesequences[$table])) {
+            return self::$tablesequences[$table];
+        }
+
+        $id = self::$sequencenextstartingid;
+
+        // If there are records, calculate the minimum id we can use.
+        // It must be bigger than the last record's id.
+        if (!empty($records)) {
+            $lastrecord = end($records);
+            $id = max($id, $lastrecord->id + 1);
+        }
+
+        self::$sequencenextstartingid = $id + 1000;
+
+        self::$tablesequences[$table] = $id;
+
+        return $id;
+    }
+
     /**
      * Reset all database sequences to initial values.
      *
@@ -428,18 +464,24 @@ abstract class testing_util {
             return;
         }
 
+        // If all starting Id's are the same, it's difficult to detect coding and testing
+        // errors that use the incorrect id in tests.  The classic case is cmid vs instance id.
+        // To reduce the chance of the coding error, we start sequences at different values where possible.
+        // In a attempt to avoid tables with existing id's we start at a high number.
+        // Reset the value each time all database sequences are reset.
+        if (defined('PHPUNIT_SEQUENCE_START') and PHPUNIT_SEQUENCE_START) {
+            self::$sequencenextstartingid = PHPUNIT_SEQUENCE_START;
+        } else {
+            self::$sequencenextstartingid = 100000;
+        }
+
         $dbfamily = $DB->get_dbfamily();
         if ($dbfamily === 'postgres') {
             $queries = array();
             $prefix = $DB->get_prefix();
             foreach ($data as $table => $records) {
                 if (isset($structure[$table]['id']) and $structure[$table]['id']->auto_increment) {
-                    if (empty($records)) {
-                        $nextid = 1;
-                    } else {
-                        $lastrecord = end($records);
-                        $nextid = $lastrecord->id + 1;
-                    }
+                    $nextid = self::get_next_sequence_starting_value($records, $table);
                     $queries[] = "ALTER SEQUENCE {$prefix}{$table}_id_seq RESTART WITH $nextid";
                 }
             }
@@ -467,16 +509,10 @@ abstract class testing_util {
             foreach ($data as $table => $records) {
                 if (isset($structure[$table]['id']) and $structure[$table]['id']->auto_increment) {
                     if (isset($sequences[$table])) {
-                        if (empty($records)) {
-                            $nextid = 1;
-                        } else {
-                            $lastrecord = end($records);
-                            $nextid = $lastrecord->id + 1;
-                        }
+                        $nextid = self::get_next_sequence_starting_value($records, $table);
                         if ($sequences[$table] != $nextid) {
                             $DB->change_database_structure("ALTER TABLE {$prefix}{$table} AUTO_INCREMENT = $nextid");
                         }
-
                     } else {
                         // some problem exists, fallback to standard code
                         $DB->get_manager()->reset_sequence($table);
@@ -522,6 +558,7 @@ abstract class testing_util {
 
         } else {
             // note: does mssql support any kind of faster reset?
+            // This also implies mssql will not use unique sequence values.
             if (is_null($empties)) {
                 $empties = self::guess_unmodified_empty_tables();
             }
@@ -601,7 +638,11 @@ abstract class testing_util {
 
         foreach ($data as $table => $records) {
             if ($borkedmysql) {
-                if (empty($records) and isset($empties[$table])) {
+                if (empty($records)) {
+                    if (!isset($empties[$table])) {
+                        // Table has been modified and is not empty.
+                        $DB->delete_records($table, null);
+                    }
                     continue;
                 }
 
@@ -623,9 +664,8 @@ abstract class testing_util {
             }
 
             if (empty($records)) {
-                if (isset($empties[$table])) {
-                    // table was not modified and is empty
-                } else {
+                if (!isset($empties[$table])) {
+                    // Table has been modified and is not empty.
                     $DB->delete_records($table, array());
                 }
                 continue;
