@@ -395,25 +395,29 @@ class core_coursecatlib_testcase extends advanced_testcase {
             'category' => $category->id,
             'idnumber' => '006-01',
             'shortname' => 'Biome Study',
-            'fullname' => '<span lang="ar" class="multilang">'.'دراسة منطقة إحيائية'.'</span><span lang="en" class="multilang">Biome Study</span>'
+            'fullname' => '<span lang="ar" class="multilang">'.'دراسة منطقة إحيائية'.'</span><span lang="en" class="multilang">Biome Study</span>',
+            'timecreated' => '1000000001'
         ));
         $course2 = $generator->create_course(array(
             'category' => $category->id,
             'idnumber' => '007-02',
             'shortname' => 'Chemistry Revision',
-            'fullname' => 'Chemistry Revision'
+            'fullname' => 'Chemistry Revision',
+            'timecreated' => '1000000002'
         ));
         $course3 = $generator->create_course(array(
             'category' => $category->id,
             'idnumber' => '007-03',
             'shortname' => 'Swiss Rolls and Sunflowers',
-            'fullname' => 'Aarkvarks guide to Swiss Rolls and Sunflowers'
+            'fullname' => 'Aarkvarks guide to Swiss Rolls and Sunflowers',
+            'timecreated' => '1000000003'
         ));
         $course4 = $generator->create_course(array(
             'category' => $category->id,
             'idnumber' => '006-04',
             'shortname' => 'Scratch',
-            'fullname' => '<a href="test.php">Basic Scratch</a>'
+            'fullname' => '<a href="test.php">Basic Scratch</a>',
+            'timecreated' => '1000000004'
         ));
         $c1 = (int)$course1->id;
         $c2 = (int)$course2->id;
@@ -427,6 +431,8 @@ class core_coursecatlib_testcase extends advanced_testcase {
         $this->assertTrue($coursecat->resort_courses('shortname'));
         $this->assertSame(array($c1, $c2, $c4, $c3), array_keys($coursecat->get_courses()));
 
+        $this->assertTrue($coursecat->resort_courses('timecreated'));
+        $this->assertSame(array($c1, $c2, $c3, $c4), array_keys($coursecat->get_courses()));
 
         try {
             // Enable the multilang filter and set it to apply to headings and content.
@@ -555,6 +561,12 @@ class core_coursecatlib_testcase extends advanced_testcase {
 
         $manual = enrol_get_plugin('manual');
 
+        // Nobody is enrolled now and course contacts are empty.
+        $allcourses = coursecat::get(0)->get_courses(array('recursive' => true, 'coursecontacts' => true, 'sort' => array('idnumber' => 1)));
+        foreach ($allcourses as $onecourse) {
+            $this->assertEmpty($onecourse->get_course_contacts());
+        }
+
         // Cat1 (user2 has teacher role)
         role_assign($teacherrole->id, $user[2], context_coursecat::instance($category[1]));
         // course21 (user2 is enrolled as manager)
@@ -610,6 +622,14 @@ class core_coursecatlib_testcase extends advanced_testcase {
         //   -- course12 (user1 has teacher role)         |
         $this->assertSame('', $contacts[1][2]);
 
+        // Suspend user 4 and make sure he is no longer in contacts of course 1 in category 4.
+        $manual->enrol_user($enrol[4][1], $user[4], $teacherrole->id, 0, 0, ENROL_USER_SUSPENDED);
+        $allcourses = coursecat::get(0)->get_courses(array('recursive' => true, 'coursecontacts' => true, 'sort' => array('idnumber' => 1)));
+        $contacts = $allcourses[$course[4][1]]->get_course_contacts();
+        $this->assertCount(1, $contacts);
+        $contact = reset($contacts);
+        $this->assertEquals('F5 L5', $contact['username']);
+
         $CFG->coursecontact = $oldcoursecontact;
     }
 
