@@ -188,7 +188,8 @@ ACTIONMENU.prototype = {
      */
     getMenuItem: function(currentItem, previous) {
         var menubar = currentItem.ancestor(SELECTOR.MENUBAR),
-            menuitems;
+            menuitems,
+            next;
 
         if (!menubar) {
             return null;
@@ -244,9 +245,10 @@ ACTIONMENU.prototype = {
 
     /**
      * Hides the menu if it is visible.
+     * @param {EventFacade} e
      * @method hideMenu
      */
-    hideMenu : function() {
+    hideMenu : function(e) {
         if (this.dialogue) {
             Y.log('Hiding an action menu', 'debug', ACTIONMENU.NAME);
             this.dialogue.removeClass('show');
@@ -265,7 +267,9 @@ ACTIONMENU.prototype = {
         }
 
         if (this.menulink) {
-            this.menulink.focus();
+            if (e.type != 'click') {
+                this.menulink.focus();
+            }
             this.menulink = null;
         }
     },
@@ -292,7 +296,7 @@ ACTIONMENU.prototype = {
 
         // Prevent event propagation as it will trigger the hideIfOutside event handler in certain situations.
         e.halt(true);
-        this.hideMenu();
+        this.hideMenu(e);
         if (menuvisible) {
             // The menu was visible and the user has clicked to toggle it again.
             return;
@@ -311,43 +315,49 @@ ACTIONMENU.prototype = {
      */
     handleKeyboardEvent: function(e) {
         var next;
+        var markEventHandled = function(e) {
+            e.preventDefault();
+            e.stopPropagation();
+        };
 
         // Handle when the menu is still selected.
         if (e.currentTarget.ancestor(SELECTOR.TOGGLE, true)) {
             if ((e.keyCode === 40 || (e.keyCode === 9 && !e.shiftKey)) && this.firstMenuChild) {
                 this.firstMenuChild.focus();
-                e.preventDefault();
+                markEventHandled(e);
             } else if (e.keyCode === 38 && this.lastMenuChild) {
                 this.lastMenuChild.focus();
-                e.preventDefault();
+                markEventHandled(e);
             } else if (e.keyCode === 9 && e.shiftKey) {
-                this.hideMenu();
-                e.preventDefault();
+                this.hideMenu(e);
+                markEventHandled(e);
             }
             return this;
         }
 
         if (e.keyCode === 27) {
             // The escape key was pressed so close the menu.
-            this.hideMenu();
-            e.preventDefault();
+            this.hideMenu(e);
+            markEventHandled(e);
 
         } else if (e.keyCode === 32) {
             // The space bar was pressed. Trigger a click.
-            e.preventDefault();
+            markEventHandled(e);
             e.currentTarget.simulate('click');
         } else if (e.keyCode === 9) {
             // The tab key was pressed. Tab moves forwards, Shift + Tab moves backwards through the menu options.
-            // We only override the Shift + Tab on the first option, and Tab on the last option to change where the focus is moved to.
+            // We only override the Shift + Tab on the first option, and Tab on the last option to change where the
+            // focus is moved to.
             if (e.target === this.firstMenuChild && e.shiftKey) {
-                this.hideMenu();
-                e.preventDefault();
+                this.hideMenu(e);
+                markEventHandled(e);
             } else if (e.target === this.lastMenuChild && !e.shiftKey) {
-                if (this.hideMenu()) {
+                if (this.hideMenu(e)) {
                     // Determine the next selector and focus on it.
                     next = this.menulink.next(SELECTOR.CAN_RECEIVE_FOCUS_SELECTOR);
                     if (next) {
                         next.focus();
+                        markEventHandled(e);
                     }
                 }
             }
@@ -396,7 +406,7 @@ ACTIONMENU.prototype = {
 
             if (next) {
                 next.focus();
-                e.preventDefault();
+                markEventHandled(e);
             }
         }
     },
@@ -409,8 +419,8 @@ ACTIONMENU.prototype = {
      * @param {EventFacade} e
      */
     hideIfOutside : function(e) {
-        if (!e.target.ancestor(SELECTOR.MENUCHILD, true)) {
-            this.hideMenu();
+        if (!e.target.ancestor(SELECTOR.MENUCONTENT, true)) {
+            this.hideMenu(e);
         }
     },
 
@@ -455,14 +465,17 @@ ACTIONMENU.prototype = {
         this.events.push(BODY.delegate('focus', this.hideIfOutside, '*', this));
 
         // Check keyboard changes.
-        this.events.push(menu.delegate('key', this.handleKeyboardEvent, 'down:9, 27, 38, 40, 32', SELECTOR.MENUCHILD + ', ' + SELECTOR.TOGGLE, this));
+        this.events.push(
+            menu.delegate('key', this.handleKeyboardEvent,
+                          'down:9, 27, 38, 40, 32', SELECTOR.MENUCHILD + ', ' + SELECTOR.TOGGLE, this)
+            );
 
         // Close the menu after a button was pushed.
         this.events.push(menu.delegate('click', function(e) {
             if (e.currentTarget.test(SELECTOR.KEEPOPEN)) {
                 return;
             }
-            this.hideMenu();
+            this.hideMenu(e);
         }, SELECTOR.MENUCHILD, this));
 
         return true;
