@@ -29,8 +29,9 @@ DIALOGUE = function(c) {
     var id = 'moodle-dialogue-' + config.COUNT;
     config.notificationBase =
         Y.Node.create('<div class="'+CSS.BASE+'">')
-              .append(Y.Node.create('<div id="'+id+'" role="dialog" aria-labelledby="'+id+'-header-text" class="'+CSS.WRAP+'"></div>')
-              .append(Y.Node.create('<div id="'+id+'-header-text" class="'+CSS.HEADER+' yui3-widget-hd"></div>'))
+              .append(Y.Node.create('<div id="' + id + '" role="dialog" ' +
+                                    'aria-labelledby="' + id + '-header-text" class="' + CSS.WRAP + '"></div>')
+              .append(Y.Node.create('<div id="' + id + '-header-text" class="'+CSS.HEADER+' yui3-widget-hd"></div>'))
               .append(Y.Node.create('<div class="'+CSS.BODY+' yui3-widget-bd"></div>'))
               .append(Y.Node.create('<div class="'+CSS.FOOTER+' yui3-widget-ft"></div>')));
     Y.one(document.body).append(config.notificationBase);
@@ -69,6 +70,26 @@ Y.extend(DIALOGUE, Y.Panel, {
     _orientationevent : null,
     _calculatedzindex : false,
 
+    /**
+     * The original position of the dialogue before it was reposition to
+     * avoid browser jumping.
+     *
+     * @property _originalPosition
+     * @protected
+     * @type Array
+     */
+    _originalPosition: null,
+
+    /**
+     * The list of elements that have been aria hidden when displaying
+     * this dialogue.
+     *
+     * @property _hiddenSiblings
+     * @protected
+     * @type Array
+     */
+    _hiddenSiblings: null,
+
     /**
      * Initialise the dialogue.
      *
@@ -77,16 +98,21 @@ Y.extend(DIALOGUE, Y.Panel, {
     initializer : function() {
         var bb;
 
+        // Initialise the element cache.
+        this._hiddenSiblings = [];
+
         if (this.get('render')) {
             this.render();
         }
-        this.makeResponsive();
         this.after('visibleChange', this.visibilityChanged, this);
         if (this.get('center')) {
             this.centerDialogue();
         }
 
         if (this.get('modal')) {
+            // If we're a modal then make sure our container is ARIA
+            // hidden by default. ARIA visibility is managed for modal dialogues.
+            this.get(BASE).set('aria-hidden', 'true');
             this.plug(Y.M.core.LockScroll);
         }
 
@@ -103,14 +129,25 @@ Y.extend(DIALOGUE, Y.Panel, {
         }
         // Recalculate the zIndex every time the modal is altered.
         this.on('maskShow', this.applyZIndex);
-        // We must show - after the dialogue has been positioned,
-        // either by centerDialogue or makeResonsive. This is because the show() will trigger
-        // a focus on the dialogue, which will scroll the page. If the dialogue has not
-        // been positioned it will scroll back to the top of the page.
-        if (this.get('visible')) {
-            this.show();
-            this.keyDelegation();
-        }
+
+        this.on('maskShow', function() {
+            // When the mask shows, position the boundingBox at the top-left of the window such that when it is
+            // focused, the position does not change.
+            var w = Y.one(Y.config.win),
+                bb = this.get('boundingBox');
+
+            if (!this.get('center')) {
+                this._originalPosition = bb.getXY();
+            }
+
+            if (bb.getStyle('position') !== 'fixed') {
+                // If the boundingBox has been positioned in a fixed manner, then it will not position correctly to scrollTop.
+                bb.setStyles({
+                    top: w.get('scrollTop'),
+                    left: w.get('scrollLeft')
+                });
+            }
+        }, this);
 
         // Remove the dialogue from the DOM when it is destroyed.
         this.after('destroyedChange', function(){
@@ -190,6 +227,7 @@ Y.extend(DIALOGUE, Y.Panel, {
         var titlebar, bb;
         if (e.attrName === 'visible') {
             this.get('maskNode').addClass(CSS.LIGHTBOX);
+            // Going from visible to hidden.
             if (e.prevVal && !e.newVal) {
                 bb = this.get('boundingBox');
                 if (this._resizeevent) {
@@ -201,7 +239,13 @@ Y.extend(DIALOGUE, Y.Panel, {
                     this._orientationevent = null;
                 }
                 bb.detach('key', this.keyDelegation);
+
+                if (this.get('modal')) {
+                    // Hide this dialogue from screen readers.
+                    this.setAccessibilityHidden();
+                }
             }
+            // Going from hidden to visible.
             if (!e.prevVal && e.newVal) {
                 // This needs to be done each time the dialog is shown as new dialogs may have been opened.
                 this.applyZIndex();
@@ -215,6 +259,13 @@ Y.extend(DIALOGUE, Y.Panel, {
                     }
                 }
                 this.keyDelegation();
+
+                // Only do accessibility hiding for modals because the ARIA spec
+                // says that all ARIA dialogues should be modal.
+                if (this.get('modal')) {
+                    // Make this dialogue visible to screen readers.
+                    this.setAccessibilityVisible();
+                }
             }
             if (this.get('center') && !e.prevVal && e.newVal) {
                 this.centerDialogue();
@@ -228,8 +279,7 @@ Y.extend(DIALOGUE, Y.Panel, {
      * @method makeResponsive
      */
     makeResponsive : function() {
-        var bb = this.get('boundingBox'),
-            content;
+        var bb = this.get('boundingBox');
 
         if (this.shouldResizeFullscreen()) {
             // Make this dialogue fullscreen on a small screen.
@@ -244,15 +294,12 @@ Y.extend(DIALOGUE, Y.Panel, {
                           'height' : null,
                           'right' : null,
                           'bottom' : null});
-
-            content = Y.one('#' + this.get('id') + ' .' + CSS.BODY);
         } else {
             if (this.get('responsive')) {
                 // We must reset any of the fullscreen changes.
                 bb.removeClass(DIALOGUE_FULLSCREEN_CLASS)
                     .setStyles({'width' : this.get('width'),
                                 'height' : this.get('height')});
-                content = Y.one('#' + this.get('id') + ' .' + CSS.BODY);
             }
         }
     },
@@ -281,6 +328,7 @@ Y.extend(DIALOGUE, Y.Panel, {
         if (hidden) {
             bb.addClass(DIALOGUE_HIDDEN_CLASS);
         }
+        this.makeResponsive();
     },
     /**
      * Return whether this dialogue should be fullscreen or not.
@@ -296,7 +344,7 @@ Y.extend(DIALOGUE, Y.Panel, {
                Math.floor(Y.one(document.body).get('winWidth')) < this.get('responsiveWidth');
     },
 
-    show : function() {
+    show: function() {
         var result = null,
             header = this.headerNode,
             content = this.bodyNode,
@@ -305,6 +353,11 @@ Y.extend(DIALOGUE, Y.Panel, {
 
         result = DIALOGUE.superclass.show.call(this);
 
+        if (!this.get('center') && this._originalPosition) {
+            // Restore the dialogue position to it's location before it was moved at show time.
+            this.get('boundingBox').setXY(this._originalPosition);
+        }
+
         // Lock scroll if the plugin is present.
         if (this.lockScroll) {
             // We need to force the scroll locking for full screen dialogues, even if they have a small vertical size to
@@ -383,6 +436,71 @@ Y.extend(DIALOGUE, Y.Panel, {
         } else if (target === firstitem && direction === 'backward') {  // Tab+shift key.
             return lastitem.focus();
         }
+    },
+
+    /**
+     * Sets the appropriate aria attributes on this dialogue and the other
+     * elements in the DOM to ensure that screen readers are able to navigate
+     * the dialogue popup correctly.
+     *
+     * @method setAccessibilityVisible
+     */
+    setAccessibilityVisible: function() {
+        // Get the element that contains this dialogue because we need it
+        // to filter out from the document.body child elements.
+        var container = this.get(BASE);
+
+        // We need to get a list containing each sibling element and the shallowest
+        // non-ancestral nodes in the DOM. We can shortcut this a little by leveraging
+        // the fact that this dialogue is always appended to the document body therefore
+        // it's siblings are the shallowest non-ancestral nodes. If that changes then
+        // this code should also be updated.
+        Y.one(document.body).get('children').each(function(node) {
+            // Skip the element that contains us.
+            if (node !== container) {
+                var hidden = node.get('aria-hidden');
+                // If they are already hidden we can ignore them.
+                if (hidden !== 'true') {
+                    // Save their current state.
+                    node.setData('previous-aria-hidden', hidden);
+                    this._hiddenSiblings.push(node);
+
+                    // Hide this node from screen readers.
+                    node.set('aria-hidden', 'true');
+                }
+            }
+        }, this);
+
+        // Make us visible to screen readers.
+        container.set('aria-hidden', 'false');
+    },
+
+    /**
+     * Restores the aria visibility on the DOM elements changed when displaying
+     * the dialogue popup and makes the dialogue aria hidden to allow screen
+     * readers to navigate the main page correctly when the dialogue is closed.
+     *
+     * @method setAccessibilityHidden
+     */
+    setAccessibilityHidden: function() {
+        var container = this.get(BASE);
+        container.set('aria-hidden', 'true');
+
+        // Restore the sibling nodes back to their original values.
+        Y.Array.each(this._hiddenSiblings, function(node) {
+            var previousValue = node.getData('previous-aria-hidden');
+            // If the element didn't previously have an aria-hidden attribute
+            // then we can just remove the one we set.
+            if (previousValue === null) {
+                node.removeAttribute('aria-hidden');
+            } else {
+                // Otherwise set it back to the old value (which will be false).
+                node.set('aria-hidden', previousValue);
+            }
+        });
+
+        // Clear the cache. No longer need to store these.
+        this._hiddenSiblings = [];
     }
 }, {
     NAME : DIALOGUE_NAME,
@@ -404,7 +522,8 @@ Y.extend(DIALOGUE, Y.Panel, {
         lightbox: {
             lazyAdd: false,
             setter: function(value) {
-                Y.log("The lightbox attribute of M.core.dialogue has been deprecated since Moodle 2.7, please use the modal attribute instead",
+                Y.log("The lightbox attribute of M.core.dialogue has been deprecated since Moodle 2.7, " +
+                      "please use the modal attribute instead",
                     'warn', 'moodle-core-notification-dialogue');
                 this.set('modal', value);
             }
