@@ -1,5 +1,5 @@
 /*
-YUI 3.15.0 (build 834026e)
+YUI 3.17.2 (build 9c3c78e)
 Copyright 2014 Yahoo! Inc. All rights reserved.
 Licensed under the BSD License.
 http://yuilibrary.com/license/
@@ -689,4 +689,4 @@ YUI.add('anim-base', function (Y, NAME) {
     Y.extend(Y.Anim, Y.Base, proto);
 
 
-}, '3.15.0', {"requires": ["base-base", "node-style"]});
+}, '3.17.2', {"requires": ["base-base", "node-style", "color-base"]});
