@@ -386,6 +386,10 @@ class assign_submission_status implements renderable {
     public $maxattempts = -1;
     /** @var string gradingstatus */
     public $gradingstatus = '';
+    /** @var bool preventsubmissionnotingroup */
+    public $preventsubmissionnotingroup = 0;
+    /** @var array usergroups */
+    public $usergroups = array();
 
 
     /**
@@ -419,6 +423,8 @@ class assign_submission_status implements renderable {
      * @param string $attemptreopenmethod - The method of reopening student attempts.
      * @param int $maxattempts - How many attempts can a student make?
      * @param string $gradingstatus - The submission status (ie. Graded, Not Released etc).
+     * @param bool $preventsubmissionnotingroup - Prevent submission if user is not in a group
+     * @param array $usergroups - Array containing all groups the user is assigned to
      */
     public function __construct($allowsubmissionsfromdate,
                                 $alwaysshowdescription,
@@ -447,7 +453,9 @@ class assign_submission_status implements renderable {
                                 $gradingcontrollerpreview,
                                 $attemptreopenmethod,
                                 $maxattempts,
-                                $gradingstatus) {
+                                $gradingstatus,
+                                $preventsubmissionnotingroup,
+                                $usergroups) {
         $this->allowsubmissionsfromdate = $allowsubmissionsfromdate;
         $this->alwaysshowdescription = $alwaysshowdescription;
         $this->submission = $submission;
@@ -476,6 +484,8 @@ class assign_submission_status implements renderable {
         $this->attemptreopenmethod = $attemptreopenmethod;
         $this->maxattempts = $maxattempts;
         $this->gradingstatus = $gradingstatus;
+        $this->preventsubmissionnotingroup = $preventsubmissionnotingroup;
+        $this->usergroups = $usergroups;
     }
 }
 
@@ -565,6 +575,8 @@ class assign_header implements renderable {
     public $subpage = '';
     /** @var string $preface optional preface (text to show before the heading) */
     public $preface = '';
+    /** @var string $postfix optional postfix (text to show after the intro) */
+    public $postfix = '';
 
     /**
      * Constructor
@@ -581,13 +593,15 @@ class assign_header implements renderable {
                                 $showintro,
                                 $coursemoduleid,
                                 $subpage='',
-                                $preface='') {
+                                $preface='',
+                                $postfix='') {
         $this->assign = $assign;
         $this->context = $context;
         $this->showintro = $showintro;
         $this->coursemoduleid = $coursemoduleid;
         $this->subpage = $subpage;
         $this->preface = $preface;
+        $this->postfix = $postfix;
     }
 }
 
@@ -638,6 +652,8 @@ class assign_grading_summary implements renderable {
     public $coursemoduleid = 0;
     /** @var boolean teamsubmission - Are team submissions enabled for this assignment */
     public $teamsubmission = false;
+    /** @var boolean warnofungroupedusers - Do we need to warn people that there are users without groups */
+    public $warnofungroupedusers = false;
 
     /**
      * constructor
@@ -662,7 +678,8 @@ class assign_grading_summary implements renderable {
                                 $duedate,
                                 $coursemoduleid,
                                 $submissionsneedgradingcount,
-                                $teamsubmission) {
+                                $teamsubmission,
+                                $warnofungroupedusers) {
         $this->participantcount = $participantcount;
         $this->submissiondraftsenabled = $submissiondraftsenabled;
         $this->submissiondraftscount = $submissiondraftscount;
@@ -673,6 +690,7 @@ class assign_grading_summary implements renderable {
         $this->coursemoduleid = $coursemoduleid;
         $this->submissionsneedgradingcount = $submissionsneedgradingcount;
         $this->teamsubmission = $teamsubmission;
+        $this->warnofungroupedusers = $warnofungroupedusers;
     }
 }
 
@@ -770,7 +788,7 @@ class assign_files implements renderable {
 
         if (!empty($CFG->enableportfolios)) {
             require_once($CFG->libdir . '/portfoliolib.php');
-            if (count($files) >= 1 &&
+            if (count($files) >= 1 && !empty($sid) &&
                     has_capability('mod/assign:exportownsubmission', $this->context)) {
                 $button = new portfolio_add_button();
                 $callbackparams = array('cmid' => $this->cm->id,
@@ -805,6 +823,7 @@ class assign_files implements renderable {
         foreach ($dir['files'] as $file) {
             $file->portfoliobutton = '';
             if (!empty($CFG->enableportfolios)) {
+                require_once($CFG->libdir . '/portfoliolib.php');
                 $button = new portfolio_add_button();
                 if (has_capability('mod/assign:exportownsubmission', $this->context)) {
                     $portfolioparams = array('cmid' => $this->cm->id, 'fileid' => $file->get_id());
