@@ -77,15 +77,34 @@ if ($page != 0 && $offset == 0) {
 /// only if the glossary is viewed by the first time
 if ( $dp = $DB->get_record('glossary_formats', array('name'=>$glossary->displayformat)) ) {
 /// Based on format->defaultmode, we build the defaulttab to be showed sometimes
+    $showtabs = glossary_get_visible_tabs($dp);
     switch ($dp->defaultmode) {
         case 'cat':
             $defaulttab = GLOSSARY_CATEGORY_VIEW;
+
+            // Handle defaultmode if 'category' tab is disabled. Fallback to 'standard' tab.
+            if (!in_array(GLOSSARY_CATEGORY, $showtabs)) {
+                $defaulttab = GLOSSARY_STANDARD_VIEW;
+            }
+
             break;
         case 'date':
             $defaulttab = GLOSSARY_DATE_VIEW;
+
+            // Handle defaultmode if 'date' tab is disabled. Fallback to 'standard' tab.
+            if (!in_array(GLOSSARY_DATE, $showtabs)) {
+                $defaulttab = GLOSSARY_STANDARD_VIEW;
+            }
+
             break;
         case 'author':
             $defaulttab = GLOSSARY_AUTHOR_VIEW;
+
+            // Handle defaultmode if 'author' tab is disabled. Fallback to 'standard' tab.
+            if (!in_array(GLOSSARY_AUTHOR, $showtabs)) {
+                $defaulttab = GLOSSARY_STANDARD_VIEW;
+            }
+
             break;
         default:
             $defaulttab = GLOSSARY_STANDARD_VIEW;
@@ -100,6 +119,7 @@ if ( $dp = $DB->get_record('glossary_formats', array('name'=>$glossary->displayf
     }
 } else {
     $defaulttab = GLOSSARY_STANDARD_VIEW;
+    $showtabs = array($defaulttab);
     $printpivot = 1;
     if ( $mode == '' and $hook == '' and $show == '') {
         $mode = 'letter';
@@ -116,14 +136,6 @@ if ( $show ) {
     $hook = $show;
     $show = '';
 }
-/// Processing standard security processes
-if ($course->id != SITEID) {
-    require_login($course);
-}
-if (!$cm->visible and !has_capability('moodle/course:viewhiddenactivities', $context)) {
-    echo $OUTPUT->header();
-    notice(get_string("activityiscurrentlyhidden"));
-}
 
 /// stablishing flag variables
 if ( $sortorder = strtolower($sortorder) ) {
@@ -159,6 +171,12 @@ break;
 
 case 'cat':    /// Looking for a certain cat
     $tab = GLOSSARY_CATEGORY_VIEW;
+
+    // Validation - we don't want to display 'category' tab if it is disabled.
+    if (!in_array(GLOSSARY_CATEGORY, $showtabs)) {
+        $tab = GLOSSARY_STANDARD_VIEW;
+    }
+
     if ( $hook > 0 ) {
         $category = $DB->get_record("glossary_categories", array("id"=>$hook));
     }
@@ -182,6 +200,12 @@ break;
 
 case 'date':
     $tab = GLOSSARY_DATE_VIEW;
+
+    // Validation - we dont want to display 'date' tab if it is disabled.
+    if (!in_array(GLOSSARY_DATE, $showtabs)) {
+        $tab = GLOSSARY_STANDARD_VIEW;
+    }
+
     if ( !$sortkey ) {
         $sortkey = 'UPDATE';
     }
@@ -192,6 +216,12 @@ break;
 
 case 'author':  /// Looking for entries, browsed by author
     $tab = GLOSSARY_AUTHOR_VIEW;
+
+    // Validation - we dont want to display 'author' tab if it is disabled.
+    if (!in_array(GLOSSARY_AUTHOR, $showtabs)) {
+        $tab = GLOSSARY_STANDARD_VIEW;
+    }
+
     if ( !$hook ) {
         $hook = 'ALL';
     }
