@@ -34,7 +34,7 @@ class mod_label_mod_form extends moodleform_mod {
         $mform = $this->_form;
 
         $mform->addElement('header', 'generalhdr', get_string('general'));
-        $this->add_intro_editor(true, get_string('labeltext', 'label'));
+        $this->standard_intro_elements(get_string('labeltext', 'label'));
 
         $this->standard_coursemodule_elements();
 
