@@ -57,6 +57,18 @@ class lesson_page_type_shortanswer extends lesson_page {
             $data->answer = s($attempt->useranswer);
         }
         $mform->set_data($data);
+
+        // Trigger an event question viewed.
+        $eventparams = array(
+            'context' => context_module::instance($PAGE->cm->id),
+            'objectid' => $this->properties->id,
+            'other' => array(
+                    'pagetype' => $this->get_typestring()
+                )
+            );
+
+        $event = \mod_lesson\event\question_viewed::create($eventparams);
+        $event->trigger();
         return $mform->display();
     }
     public function check_answer() {
@@ -76,6 +88,7 @@ class lesson_page_type_shortanswer extends lesson_page {
         $i=0;
         $answers = $this->get_answers();
         foreach ($answers as $answer) {
+            $answer = parent::rewrite_answers_urls($answer, false);
             $i++;
             // Applying PARAM_TEXT as it is applied to the answer submitted by the user.
             $expectedanswer  = clean_param($answer->answer, PARAM_TEXT);
@@ -155,9 +168,9 @@ class lesson_page_type_shortanswer extends lesson_page {
             }
             if ($ismatch) {
                 $result->newpageid = $answer->jumpto;
-                if (trim(strip_tags($answer->response))) {
-                    $result->response = $answer->response;
-                }
+                $options = new stdClass();
+                $options->para = false;
+                $result->response = format_text($answer->response, $answer->responseformat, $options);
                 $result->answerid = $answer->id;
                 break; // quit answer analysis immediately after a match has been found
             }
@@ -182,6 +195,7 @@ class lesson_page_type_shortanswer extends lesson_page {
         $options->para = false;
         $i = 1;
         foreach ($answers as $answer) {
+            $answer = parent::rewrite_answers_urls($answer, false);
             $cells = array();
             if ($this->lesson->custom && $answer->score > 0) {
                 // if the score is > 0, then it is correct
@@ -239,10 +253,13 @@ class lesson_page_type_shortanswer extends lesson_page {
     }
 
     public function report_answers($answerpage, $answerdata, $useranswer, $pagestats, &$i, &$n) {
+        global $PAGE;
+
         $answers = $this->get_answers();
         $formattextdefoptions = new stdClass;
         $formattextdefoptions->para = false;  //I'll use it widely in this page
         foreach ($answers as $answer) {
+            $answer = parent::rewrite_answers_urls($answer, false);
             if ($useranswer == null && $i == 0) {
                 // I have the $i == 0 because it is easier to blast through it all at once.
                 if (isset($pagestats[$this->properties->id])) {
@@ -311,6 +328,8 @@ class lesson_page_type_shortanswer extends lesson_page {
 class lesson_add_page_form_shortanswer extends lesson_add_page_form_base {
     public $qtype = 'shortanswer';
     public $qtypestring = 'shortanswer';
+    protected $answerformat = '';
+    protected $responseformat = LESSON_ANSWER_HTML;
 
     public function custom_definition() {
 
@@ -346,24 +365,34 @@ class lesson_display_answer_form_shortanswer extends moodleform {
             }
         }
 
+        $placeholder = false;
+        if (preg_match('/_____+/', $contents, $matches)) {
+            $placeholder = $matches[0];
+            $contentsparts = explode( $placeholder, $contents, 2);
+            $attrs['size'] = round(strlen($placeholder) * 1.1);
+        }
+
         // Disable shortforms.
         $mform->setDisableShortforms();
 
         $mform->addElement('header', 'pageheader');
-
-        $mform->addElement('html', $OUTPUT->container($contents, 'contents'));
-
-        $options = new stdClass;
-        $options->para = false;
-        $options->noclean = true;
-
         $mform->addElement('hidden', 'id');
         $mform->setType('id', PARAM_INT);
 
         $mform->addElement('hidden', 'pageid');
         $mform->setType('pageid', PARAM_INT);
 
-        $mform->addElement('text', 'answer', get_string('youranswer', 'lesson'), $attrs);
+        if ($placeholder) {
+            $contentsgroup = array();
+            $contentsgroup[] = $mform->createElement('static', '', '', $contentsparts[0]);
+            $contentsgroup[] = $mform->createElement('text', 'answer', '', $attrs);
+            $contentsgroup[] = $mform->createElement('static', '', '', $contentsparts[1]);
+            $mform->addGroup($contentsgroup, '', '', '', false);
+        } else {
+            $mform->addElement('html', $OUTPUT->container($contents, 'contents'));
+            $mform->addElement('text', 'answer', get_string('youranswer', 'lesson'), $attrs);
+
+        }
         $mform->setType('answer', PARAM_TEXT);
 
         if ($hasattempt) {
