@@ -54,6 +54,19 @@
                 updateToolMatches();
 
                 self.toggleEditButtons();
+
+                if (self.getSelectedToolTypeOption().getAttribute('toolproxy')){
+                    var allowname = Y.one('#id_instructorchoicesendname');
+                    allowname.set('checked', !self.getSelectedToolTypeOption().getAttribute('noname'));
+
+                    var allowemail = Y.one('#id_instructorchoicesendemailaddr');
+                    allowemail.set('checked', !self.getSelectedToolTypeOption().getAttribute('noemail'));
+
+                    var allowgrades = Y.one('#id_instructorchoiceacceptgrades');
+                    allowgrades.set('checked', !self.getSelectedToolTypeOption().getAttribute('nogrades'));
+                    self.toggleGradeSection();
+                }
+
             });
 
             this.createTypeEditorButtons();
@@ -77,9 +90,25 @@
                 }, 2000);
             });
 
+            var allowgrades = Y.one('#id_instructorchoiceacceptgrades');
+            allowgrades.on('change', this.toggleGradeSection, this);
+
             updateToolMatches();
         },
 
+        toggleGradeSection: function(e) {
+            if (e) {
+                e.preventDefault();
+            }
+            var allowgrades = Y.one('#id_instructorchoiceacceptgrades');
+            var gradefieldset = Y.one('#id_modstandardgrade');
+            if (!allowgrades.get('checked')) {
+                gradefieldset.hide();
+            } else {
+                gradefieldset.show();
+            }
+        },
+
         clearToolCache: function(){
             this.urlCache = {};
             this.toolTypeCache = {};
@@ -122,10 +151,10 @@
                 var domainRegex = /(?:https?:\/\/)?(?:www\.)?([^\/]+)(?:\/|$)/i;
                 var match = domainRegex.exec(url);
                 if(match && match[1] && match[1].toLowerCase() === selectedOption.getAttribute('domain').toLowerCase()){
-                    automatchToolDisplay.set('innerHTML',  '<img style="vertical-align:text-bottom" src="' + self.settings.green_check_icon_url + '" />' + M.str.lti.using_tool_configuration + selectedOption.get('text'));
+                    automatchToolDisplay.set('innerHTML',  '<img style="vertical-align:text-bottom" src="' + self.settings.green_check_icon_url + '" />' + M.util.get_string('using_tool_configuration', 'lti') + selectedOption.get('text'));
                 } else {
                     // The entered URL does not match the domain of the tool configuration
-                    automatchToolDisplay.set('innerHTML', '<img style="vertical-align:text-bottom" src="' + self.settings.warning_icon_url + '" />' + M.str.lti.domain_mismatch);
+                    automatchToolDisplay.set('innerHTML', '<img style="vertical-align:text-bottom" src="' + self.settings.warning_icon_url + '" />' + M.util.get_string('domain_mismatch', 'lti'));
                 }
             }
 
@@ -135,18 +164,19 @@
             // Indicate the tool is manually configured
             // We still check the Launch URL with the server as course/site tools may override privacy settings
             if(key.get('value') !== '' && secret.get('value') !== ''){
-                automatchToolDisplay.set('innerHTML',  '<img style="vertical-align:text-bottom" src="' + self.settings.green_check_icon_url + '" />' + M.str.lti.custom_config);
+                automatchToolDisplay.set('innerHTML',  '<img style="vertical-align:text-bottom" src="' + self.settings.green_check_icon_url + '" />' + M.util.get_string('custom_config', 'lti'));
             }
 
-            var continuation = function(toolInfo){
-                self.updatePrivacySettings(toolInfo);
-
+            var continuation = function(toolInfo, inputfield){
+                if (inputfield === undefined || (inputfield.get('id') != 'id_securetoolurl' || inputfield.get('value'))) {
+                    self.updatePrivacySettings(toolInfo);
+                }
                 if(toolInfo.toolname){
-                    automatchToolDisplay.set('innerHTML',  '<img style="vertical-align:text-bottom" src="' + self.settings.green_check_icon_url + '" />' + M.str.lti.using_tool_configuration + toolInfo.toolname);
+                    automatchToolDisplay.set('innerHTML',  '<img style="vertical-align:text-bottom" src="' + self.settings.green_check_icon_url + '" />' + M.util.get_string('using_tool_configuration', 'lti') + toolInfo.toolname);
                 } else if(!selectedToolType) {
                     // Inform them custom configuration is in use
                     if(key.get('value') === '' || secret.get('value') === ''){
-                        automatchToolDisplay.set('innerHTML', '<img style="vertical-align:text-bottom" src="' + self.settings.warning_icon_url + '" />' + M.str.lti.tool_config_not_found);
+                        automatchToolDisplay.set('innerHTML', '<img style="vertical-align:text-bottom" src="' + self.settings.warning_icon_url + '" />' + M.util.get_string('tool_config_not_found', 'lti'));
                     }
                 }
             };
@@ -159,7 +189,7 @@
                 return continuation(self.urlCache[url]);
             } else if(!selectedToolType && !url) {
                 // No tool type or url set
-                return continuation({});
+                return continuation({}, field);
             } else {
                 self.findToolByUrl(url, selectedToolType, function(toolInfo){
                     if(toolInfo){
@@ -223,11 +253,11 @@
                     if(settingValue == M.mod_lti.LTI_SETTING_NEVER){
                         control.set('disabled', true);
                         control.set('checked', false);
-                        control.set('title', M.str.lti.forced_help);
+                        control.set('title', M.util.get_string('forced_help', 'lti'));
                     } else if(settingValue == M.mod_lti.LTI_SETTING_ALWAYS){
                         control.set('disabled', true);
                         control.set('checked', true);
-                        control.set('title', M.str.lti.forced_help);
+                        control.set('title', M.util.get_string('forced_help', 'lti'));
                     } else if(settingValue == M.mod_lti.LTI_SETTING_DELEGATE){
                         control.set('disabled', false);
 
@@ -237,6 +267,8 @@
                     }
                 }
             }
+
+            this.toggleGradeSection();
         },
 
         getSelectedToolTypeOption: function(){
@@ -257,11 +289,11 @@
 
                 var globalGroup = Y.Node.create('<optgroup />')
                                     .set('id', 'global_tool_group')
-                                    .set('label', M.str.lti.global_tool_types);
+                                    .set('label', M.util.get_string('global_tool_types', 'lti'));
 
                 var courseGroup = Y.Node.create('<optgroup />')
                                     .set('id', 'course_tool_group')
-                                    .set('label', M.str.lti.course_tool_types);
+                                    .set('label', M.util.get_string('course_tool_types', 'lti'));
 
                 var globalOptions = typeSelector.all('option[globalTool=1]').remove().each(function(node){
                     globalGroup.append(node);
@@ -299,9 +331,9 @@
                         .append(Y.Node.create('<img src="' + iconUrl + '" />'));
             }
 
-            var addIcon = createIcon('lti_add_tool_type', M.str.lti.addtype, this.settings.add_icon_url);
-            var editIcon = createIcon('lti_edit_tool_type', M.str.lti.edittype, this.settings.edit_icon_url);
-            var deleteIcon  = createIcon('lti_delete_tool_type', M.str.lti.deletetype, this.settings.delete_icon_url);
+            var addIcon = createIcon('lti_add_tool_type', M.util.get_string('addtype', 'lti'), this.settings.add_icon_url);
+            var editIcon = createIcon('lti_edit_tool_type', M.util.get_string('edittype', 'lti'), this.settings.edit_icon_url);
+            var deleteIcon  = createIcon('lti_delete_tool_type', M.util.get_string('deletetype', 'lti'), this.settings.delete_icon_url);
 
             editIcon.on('click', function(e){
                 var toolTypeId = typeSelector.get('value');
@@ -309,7 +341,7 @@
                 if(self.getSelectedToolTypeOption().getAttribute('editable')){
                     window.open(self.settings.instructor_tool_type_edit_url + '&action=edit&typeid=' + toolTypeId, 'edit_tool');
                 } else {
-                    alert(M.str.lti.cannot_edit);
+                    alert(M.util.get_string('cannot_edit', 'lti'));
                 }
             });
 
@@ -321,11 +353,11 @@
                 var toolTypeId = typeSelector.get('value');
 
                 if(self.getSelectedToolTypeOption().getAttribute('editable')){
-                    if(confirm(M.str.lti.delete_confirmation)){
+                    if(confirm(M.util.get_string('delete_confirmation', 'lti'))){
                         self.deleteTool(toolTypeId);
                     }
                 } else {
-                    alert(M.str.lti.cannot_delete);
+                    alert(M.util.get_string('cannot_delete', 'lti'));
                 }
             });
 
