@@ -33,7 +33,6 @@ function resource_supports($feature) {
         case FEATURE_MOD_ARCHETYPE:           return MOD_ARCHETYPE_RESOURCE;
         case FEATURE_GROUPS:                  return false;
         case FEATURE_GROUPINGS:               return false;
-        case FEATURE_GROUPMEMBERSONLY:        return true;
         case FEATURE_MOD_INTRO:               return true;
         case FEATURE_COMPLETION_TRACKS_VIEWS: return true;
         case FEATURE_GRADE_HAS_GRADE:         return false;
@@ -155,6 +154,9 @@ function resource_set_display_options($data) {
     if (!empty($data->showtype)) {
         $displayoptions['showtype'] = 1;
     }
+    if (!empty($data->showdate)) {
+        $displayoptions['showdate'] = 1;
+    }
     $data->displayoptions = serialize($displayoptions);
 }
 
@@ -235,8 +237,16 @@ function resource_get_coursemodule_info($coursemodule) {
 
     }
 
-    // If any optional extra details are turned on, store in custom data
-    $info->customdata = resource_get_optional_details($resource, $coursemodule);
+    // If any optional extra details are turned on, store in custom data,
+    // add some file details as well to be used later by resource_get_optional_details() without retriving.
+    // Do not store filedetails if this is a reference - they will still need to be retrieved every time.
+    if (($filedetails = resource_get_file_details($resource, $coursemodule)) && empty($filedetails['isref'])) {
+        $displayoptions = @unserialize($resource->displayoptions);
+        $displayoptions['filedetails'] = $filedetails;
+        $info->customdata = serialize($displayoptions);
+    } else {
+        $info->customdata = $resource->displayoptions;
+    }
 
     return $info;
 }
@@ -248,7 +258,11 @@ function resource_get_coursemodule_info($coursemodule) {
  * @param cm_info $cm Course module information
  */
 function resource_cm_info_view(cm_info $cm) {
-    $details = $cm->customdata;
+    global $CFG;
+    require_once($CFG->dirroot . '/mod/resource/locallib.php');
+
+    $resource = (object)array('displayoptions' => $cm->customdata);
+    $details = resource_get_optional_details($resource, $cm);
     if ($details) {
         $cm->set_after_link(' ' . html_writer::tag('span', $details,
                 array('class' => 'resourcelinkdetails')));
@@ -384,7 +398,7 @@ function resource_pluginfile($course, $cm, $context, $filearea, $args, $forcedow
 
     // should we apply filters?
     $mimetype = $file->get_mimetype();
-    if ($mimetype === 'text/html' or $mimetype === 'text/plain') {
+    if ($mimetype === 'text/html' or $mimetype === 'text/plain' or $mimetype === 'application/xhtml+xml') {
         $filter = $DB->get_field('resource', 'filterfiles', array('id'=>$cm->instance));
         $CFG->embeddedsoforcelinktarget = true;
     } else {
@@ -472,7 +486,36 @@ function resource_dndupload_handle($uploadinfo) {
     $data->printintro = $config->printintro;
     $data->showsize = (isset($config->showsize)) ? $config->showsize : 0;
     $data->showtype = (isset($config->showtype)) ? $config->showtype : 0;
+    $data->showdate = (isset($config->showdate)) ? $config->showdate : 0;
     $data->filterfiles = $config->filterfiles;
 
     return resource_add_instance($data, null);
 }
+
+/**
+ * Mark the activity completed (if required) and trigger the course_module_viewed event.
+ *
+ * @param  stdClass $resource   resource object
+ * @param  stdClass $course     course object
+ * @param  stdClass $cm         course module object
+ * @param  stdClass $context    context object
+ * @since Moodle 3.0
+ */
+function resource_view($resource, $course, $cm, $context) {
+
+    // Trigger course_module_viewed event.
+    $params = array(
+        'context' => $context,
+        'objectid' => $resource->id
+    );
+
+    $event = \mod_resource\event\course_module_viewed::create($params);
+    $event->add_record_snapshot('course_modules', $cm);
+    $event->add_record_snapshot('course', $course);
+    $event->add_record_snapshot('resource', $resource);
+    $event->trigger();
+
+    // Completion.
+    $completion = new completion_info($course);
+    $completion->set_module_viewed($cm);
+}
