@@ -39,7 +39,7 @@ define('SCORM_NAV_DISABLED', 0);
 define('SCORM_NAV_UNDER_CONTENT', 1);
 define('SCORM_NAV_FLOATING', 2);
 
-//used to check what SCORM version is being used.
+// Used to check what SCORM version is being used.
 define('SCORM_12', 1);
 define('SCORM_13', 2);
 define('SCORM_AICC', 3);
@@ -58,14 +58,14 @@ define('SCORM_DISPLAY_ATTEMPTSTATUS_ENTRY', 3);
  * @param   bool    $with_strings   (optional)
  * @return  array
  */
-function scorm_status_options($with_strings = false) {
-    // Id's are important as they are bits
+function scorm_status_options($withstrings = false) {
+    // Id's are important as they are bits.
     $options = array(
         2 => 'passed',
         4 => 'completed'
     );
 
-    if ($with_strings) {
+    if ($withstrings) {
         foreach ($options as $key => $value) {
             $options[$key] = get_string('completionstatus_'.$value, 'scorm');
         }
@@ -119,10 +119,10 @@ function scorm_add_instance($scorm, $mform=null) {
     $id = $DB->insert_record('scorm', $scorm);
 
     // Update course module record - from now on this instance properly exists and all function may be used.
-    $DB->set_field('course_modules', 'instance', $id, array('id'=>$cmid));
+    $DB->set_field('course_modules', 'instance', $id, array('id' => $cmid));
 
     // Reload scorm instance.
-    $record = $DB->get_record('scorm', array('id'=>$id));
+    $record = $DB->get_record('scorm', array('id' => $id));
 
     // Store the package and verify.
     if ($record->scormtype === SCORM_TYPE_LOCAL) {
@@ -238,9 +238,9 @@ function scorm_update_instance($scorm, $mform=null) {
 
     $DB->update_record('scorm', $scorm);
 
-    $scorm = $DB->get_record('scorm', array('id'=>$scorm->id));
+    $scorm = $DB->get_record('scorm', array('id' => $scorm->id));
 
-    /// extra fields required in grade related functions
+    // Extra fields required in grade related functions.
     $scorm->course   = $courseid;
     $scorm->idnumber = $cmidnumber;
     $scorm->cmid     = $cmid;
@@ -266,25 +266,25 @@ function scorm_update_instance($scorm, $mform=null) {
 function scorm_delete_instance($id) {
     global $CFG, $DB;
 
-    if (! $scorm = $DB->get_record('scorm', array('id'=>$id))) {
+    if (! $scorm = $DB->get_record('scorm', array('id' => $id))) {
         return false;
     }
 
     $result = true;
 
-    // Delete any dependent records
-    if (! $DB->delete_records('scorm_scoes_track', array('scormid'=>$scorm->id))) {
+    // Delete any dependent records.
+    if (! $DB->delete_records('scorm_scoes_track', array('scormid' => $scorm->id))) {
         $result = false;
     }
-    if ($scoes = $DB->get_records('scorm_scoes', array('scorm'=>$scorm->id))) {
+    if ($scoes = $DB->get_records('scorm_scoes', array('scorm' => $scorm->id))) {
         foreach ($scoes as $sco) {
-            if (! $DB->delete_records('scorm_scoes_data', array('scoid'=>$sco->id))) {
+            if (! $DB->delete_records('scorm_scoes_data', array('scoid' => $sco->id))) {
                 $result = false;
             }
         }
-        $DB->delete_records('scorm_scoes', array('scorm'=>$scorm->id));
+        $DB->delete_records('scorm_scoes', array('scorm' => $scorm->id));
     }
-    if (! $DB->delete_records('scorm', array('id'=>$scorm->id))) {
+    if (! $DB->delete_records('scorm', array('id' => $scorm->id))) {
         $result = false;
     }
 
@@ -338,9 +338,9 @@ function scorm_user_outline($course, $user, $mod, $scorm) {
         $result = new stdClass();
         $result->info = get_string('grade') . ': '. $grade->str_long_grade;
 
-        //datesubmitted == time created. dategraded == time modified or time overridden
-        //if grade was last modified by the user themselves use date graded. Otherwise use date submitted
-        //TODO: move this copied & pasted code somewhere in the grades API. See MDL-26704
+        // Datesubmitted == time created. dategraded == time modified or time overridden
+        // if grade was last modified by the user themselves use date graded. Otherwise use date submitted.
+        // TODO: move this copied & pasted code somewhere in the grades API. See MDL-26704.
         if ($grade->usermodified == $user->id || empty($grade->datesubmitted)) {
             $result->time = $grade->dategraded;
         } else {
@@ -375,7 +375,7 @@ function scorm_user_complete($course, $user, $mod, $scorm) {
     $sometoreport = false;
     $report = '';
 
-    // First Access and Last Access dates for SCOs
+    // First Access and Last Access dates for SCOs.
     require_once($CFG->dirroot.'/mod/scorm/locallib.php');
     $timetracks = scorm_get_sco_runtime($scorm->id, false, $user->id);
     $firstmodify = $timetracks->start;
@@ -401,52 +401,54 @@ function scorm_user_complete($course, $user, $mod, $scorm) {
             $org->identifier = '';
             $orgs[] = $org;
         }
-        $report .= '<div class="mod-scorm">'."\n";
+        $report .= html_writer::start_div('mod-scorm');
         foreach ($orgs as $org) {
             $conditions = array();
             $currentorg = '';
             if (!empty($org->identifier)) {
-                $report .= '<div class="orgtitle">'.$org->title.'</div>';
+                $report .= html_writer::div($org->title, 'orgtitle');
                 $currentorg = $org->identifier;
                 $conditions['organization'] = $currentorg;
             }
-            $report .= "<ul id='0' class='$liststyle'>";
+            $report .= html_writer::start_tag('ul', array('id' => '0', 'class' => $liststyle));
                 $conditions['scorm'] = $scorm->id;
             if ($scoes = $DB->get_records('scorm_scoes', $conditions, "sortorder, id")) {
-                // drop keys so that we can access array sequentially
+                // Drop keys so that we can access array sequentially.
                 $scoes = array_values($scoes);
-                $level=0;
-                $sublist=1;
-                $parents[$level]='/';
+                $level = 0;
+                $sublist = 1;
+                $parents[$level] = '/';
                 foreach ($scoes as $pos => $sco) {
-                    if ($parents[$level]!=$sco->parent) {
-                        if ($level>0 && $parents[$level-1]==$sco->parent) {
-                            $report .= "\t\t</ul></li>\n";
+                    if ($parents[$level] != $sco->parent) {
+                        if ($level > 0 && $parents[$level - 1] == $sco->parent) {
+                            $report .= html_writer::end_tag('ul').html_writer::end_tag('li');
                             $level--;
                         } else {
                             $i = $level;
                             $closelist = '';
                             while (($i > 0) && ($parents[$level] != $sco->parent)) {
-                                $closelist .= "\t\t</ul></li>\n";
+                                $closelist .= html_writer::end_tag('ul').html_writer::end_tag('li');
                                 $i--;
                             }
                             if (($i == 0) && ($sco->parent != $currentorg)) {
-                                $report .= "\t\t<li><ul id='$sublist' class='$liststyle'>\n";
+                                $report .= html_writer::start_tag('li');
+                                $report .= html_writer::start_tag('ul', array('id' => $sublist, 'class' => $liststyle));
                                 $level++;
                             } else {
                                 $report .= $closelist;
                                 $level = $i;
                             }
-                            $parents[$level]=$sco->parent;
+                            $parents[$level] = $sco->parent;
                         }
                     }
-                    $report .= "\t\t<li>";
-                    if (isset($scoes[$pos+1])) {
-                        $nextsco = $scoes[$pos+1];
+                    $report .= html_writer::start_tag('li');
+                    if (isset($scoes[$pos + 1])) {
+                        $nextsco = $scoes[$pos + 1];
                     } else {
                         $nextsco = false;
                     }
-                    if (($nextsco !== false) && ($sco->parent != $nextsco->parent) && (($level==0) || (($level>0) && ($nextsco->parent == $sco->identifier)))) {
+                    if (($nextsco !== false) && ($sco->parent != $nextsco->parent) &&
+                            (($level == 0) || (($level > 0) && ($nextsco->parent == $sco->identifier)))) {
                         $sublist++;
                     } else {
                         $report .= $OUTPUT->spacer(array("height" => "12", "width" => "13"));
@@ -455,52 +457,56 @@ function scorm_user_complete($course, $user, $mod, $scorm) {
                     if ($sco->launch) {
                         $score = '';
                         $totaltime = '';
-                        if ($usertrack=scorm_get_tracks($sco->id, $user->id)) {
+                        if ($usertrack = scorm_get_tracks($sco->id, $user->id)) {
                             if ($usertrack->status == '') {
                                 $usertrack->status = 'notattempted';
                             }
                             $strstatus = get_string($usertrack->status, 'scorm');
-                            $report .= "<img src='".$OUTPUT->pix_url($usertrack->status, 'scorm')."' alt='$strstatus' title='$strstatus' />";
+                            $report .= html_writer::img($OUTPUT->pix_url($usertrack->status, 'scorm'),
+                                                        $strstatus, array('title' => $strstatus));
                         } else {
                             if ($sco->scormtype == 'sco') {
-                                $report .= '<img src="'.$OUTPUT->pix_url('notattempted', 'scorm').'" alt="'.get_string('notattempted', 'scorm').'" title="'.get_string('notattempted', 'scorm').'" />';
+                                $report .= html_writer::img($OUTPUT->pix_url('notattempted', 'scorm'),
+                                                            get_string('notattempted', 'scorm'),
+                                                            array('title' => get_string('notattempted', 'scorm')));
                             } else {
-                                $report .= '<img src="'.$OUTPUT->pix_url('asset', 'scorm').'" alt="'.get_string('asset', 'scorm').'" title="'.get_string('asset', 'scorm').'" />';
+                                $report .= html_writer::img($OUTPUT->pix_url('asset', 'scorm'), get_string('asset', 'scorm'),
+                                                            array('title' => get_string('asset', 'scorm')));
                             }
                         }
-                        $report .= "&nbsp;$sco->title $score$totaltime</li>\n";
+                        $report .= "&nbsp;$sco->title $score$totaltime".html_writer::end_tag('li');
                         if ($usertrack !== false) {
                             $sometoreport = true;
-                            $report .= "\t\t\t<li><ul class='$liststyle'>\n";
+                            $report .= html_writer::start_tag('li').html_writer::start_tag('ul', array('class' => $liststyle));
                             foreach ($usertrack as $element => $value) {
                                 if (substr($element, 0, 3) == 'cmi') {
-                                    $report .= '<li>'.$element.' => '.s($value).'</li>';
+                                    $report .= html_writer::tag('li', $element.' => '.s($value));
                                 }
                             }
-                            $report .= "\t\t\t</ul></li>\n";
+                            $report .= html_writer::end_tag('ul').html_writer::end_tag('li');
                         }
                     } else {
-                        $report .= "&nbsp;$sco->title</li>\n";
+                        $report .= "&nbsp;$sco->title".html_writer::end_tag('li');
                     }
                 }
-                for ($i=0; $i<$level; $i++) {
-                    $report .= "\t\t</ul></li>\n";
+                for ($i = 0; $i < $level; $i++) {
+                    $report .= html_writer::end_tag('ul').html_writer::end_tag('li');
                 }
             }
-            $report .= "\t</ul><br />\n";
+            $report .= html_writer::end_tag('ul').html_writer::empty_tag('br');
         }
-        $report .= "</div>\n";
+        $report .= html_writer::end_div();
     }
     if ($sometoreport) {
         if ($firstmodify < $now) {
             $timeago = format_time($now - $firstmodify);
-            echo get_string('firstaccess', 'scorm').': '.userdate($firstmodify).' ('.$timeago.")<br />\n";
+            echo get_string('firstaccess', 'scorm').': '.userdate($firstmodify).' ('.$timeago.")".html_writer::empty_tag('br');
         }
         if ($lastmodify > 0) {
             $timeago = format_time($now - $lastmodify);
-            echo get_string('lastaccess', 'scorm').': '.userdate($lastmodify).' ('.$timeago.")<br />\n";
+            echo get_string('lastaccess', 'scorm').': '.userdate($lastmodify).' ('.$timeago.")".html_writer::empty_tag('br');
         }
-        echo get_string('report', 'scorm').":<br />\n";
+        echo get_string('report', 'scorm').":".html_writer::empty_tag('br');
         echo $report;
     } else {
         print_string('noactivity', 'scorm');
@@ -523,7 +529,7 @@ function scorm_cron () {
 
     require_once($CFG->dirroot.'/mod/scorm/locallib.php');
 
-    $sitetimezone = $CFG->timezone;
+    $sitetimezone = core_date::get_server_timezone();
     // Now see if there are any scorm updates to be done.
 
     if (!isset($CFG->scorm_updatetimelast)) {    // To catch the first time.
@@ -547,7 +553,7 @@ function scorm_cron () {
         // Now clear out AICC session table with old session data.
         $cfgscorm = get_config('scorm');
         if (!empty($cfgscorm->allowaicchacp)) {
-            $expiretime = time() - ($cfgscorm->aicchacpkeepsessiondata*24*60*60);
+            $expiretime = time() - ($cfgscorm->aicchacpkeepsessiondata * 24 * 60 * 60);
             $DB->delete_records_select('scorm_aicc_session', 'timemodified < ?', array($expiretime));
         }
     }
@@ -570,7 +576,9 @@ function scorm_get_user_grades($scorm, $userid=0) {
 
     $grades = array();
     if (empty($userid)) {
-        if ($scousers = $DB->get_records_select('scorm_scoes_track', "scormid=? GROUP BY userid", array($scorm->id), "", "userid,null")) {
+        $scousers = $DB->get_records_select('scorm_scoes_track', "scormid=? GROUP BY userid",
+                                            array($scorm->id), "", "userid,null");
+        if ($scousers) {
             foreach ($scousers as $scouser) {
                 $grades[$scouser->userid] = new stdClass();
                 $grades[$scouser->userid]->id         = $scouser->userid;
@@ -582,8 +590,10 @@ function scorm_get_user_grades($scorm, $userid=0) {
         }
 
     } else {
-        if (!$DB->get_records_select('scorm_scoes_track', "scormid=? AND userid=? GROUP BY userid", array($scorm->id, $userid), "", "userid,null")) {
-            return false; //no attempt yet
+        $preattempt = $DB->get_records_select('scorm_scoes_track', "scormid=? AND userid=? GROUP BY userid",
+                                                array($scorm->id, $userid), "", "userid,null");
+        if (!$preattempt) {
+            return false; // No attempt yet.
         }
         $grades[$userid] = new stdClass();
         $grades[$userid]->id         = $userid;
@@ -609,50 +619,20 @@ function scorm_update_grades($scorm, $userid=0, $nullifnone=true) {
 
     if ($grades = scorm_get_user_grades($scorm, $userid)) {
         scorm_grade_item_update($scorm, $grades);
-        //set complete
+        // Set complete.
         scorm_set_completion($scorm, $userid, COMPLETION_COMPLETE, $grades);
     } else if ($userid and $nullifnone) {
         $grade = new stdClass();
         $grade->userid   = $userid;
         $grade->rawgrade = null;
         scorm_grade_item_update($scorm, $grade);
-        //set incomplete.
+        // Set incomplete.
         scorm_set_completion($scorm, $userid, COMPLETION_INCOMPLETE);
     } else {
         scorm_grade_item_update($scorm);
     }
 }
 
-/**
- * Update all grades in gradebook.
- *
- * @global object
- */
-function scorm_upgrade_grades() {
-    global $DB;
-
-    $sql = "SELECT COUNT('x')
-              FROM {scorm} s, {course_modules} cm, {modules} m
-             WHERE m.name='scorm' AND m.id=cm.module AND cm.instance=s.id";
-    $count = $DB->count_records_sql($sql);
-
-    $sql = "SELECT s.*, cm.idnumber AS cmidnumber, s.course AS courseid
-              FROM {scorm} s, {course_modules} cm, {modules} m
-             WHERE m.name='scorm' AND m.id=cm.module AND cm.instance=s.id";
-    $rs = $DB->get_recordset_sql($sql);
-    if ($rs->valid()) {
-        $pbar = new progress_bar('scormupgradegrades', 500, true);
-        $i=0;
-        foreach ($rs as $scorm) {
-            $i++;
-            upgrade_set_timeout(60*5); // set up timeout, may also abort execution
-            scorm_update_grades($scorm, 0, false);
-            $pbar->update($i, $count, "Updating Scorm grades ($i/$count).");
-        }
-    }
-    $rs->close();
-}
-
 /**
  * Update/create grade item for given scorm
  *
@@ -666,17 +646,19 @@ function scorm_upgrade_grades() {
 function scorm_grade_item_update($scorm, $grades=null) {
     global $CFG, $DB;
     require_once($CFG->dirroot.'/mod/scorm/locallib.php');
-    if (!function_exists('grade_update')) { //workaround for buggy PHP versions
+    if (!function_exists('grade_update')) { // Workaround for buggy PHP versions.
         require_once($CFG->libdir.'/gradelib.php');
     }
 
-    $params = array('itemname'=>$scorm->name);
+    $params = array('itemname' => $scorm->name);
     if (isset($scorm->cmidnumber)) {
         $params['idnumber'] = $scorm->cmidnumber;
     }
 
     if ($scorm->grademethod == GRADESCOES) {
-        if ($maxgrade = $DB->count_records_select('scorm_scoes', 'scorm = ? AND '.$DB->sql_isnotempty('scorm_scoes', 'launch', false, true), array($scorm->id))) {
+        $maxgrade = $DB->count_records_select('scorm_scoes', 'scorm = ? AND '.
+                                                $DB->sql_isnotempty('scorm_scoes', 'launch', false, true), array($scorm->id));
+        if ($maxgrade) {
             $params['gradetype'] = GRADE_TYPE_VALUE;
             $params['grademax']  = $maxgrade;
             $params['grademin']  = 0;
@@ -689,7 +671,7 @@ function scorm_grade_item_update($scorm, $grades=null) {
         $params['grademin']  = 0;
     }
 
-    if ($grades  === 'reset') {
+    if ($grades === 'reset') {
         $params['reset'] = true;
         $grades = null;
     }
@@ -708,7 +690,7 @@ function scorm_grade_item_delete($scorm) {
     global $CFG;
     require_once($CFG->libdir.'/gradelib.php');
 
-    return grade_update('mod/scorm', $scorm->course, 'mod', 'scorm', $scorm->id, 0, null, array('deleted'=>1));
+    return grade_update('mod/scorm', $scorm->course, 'mod', 'scorm', $scorm->id, 0, null, array('deleted' => 1));
 }
 
 /**
@@ -744,12 +726,12 @@ function scorm_get_post_actions() {
  * @return object $scorm
  */
 function scorm_option2text($scorm) {
-    $scorm_popoup_options = scorm_get_popup_options_array();
+    $scormpopoupoptions = scorm_get_popup_options_array();
 
     if (isset($scorm->popup)) {
         if ($scorm->popup == 1) {
             $optionlist = array();
-            foreach ($scorm_popoup_options as $name => $option) {
+            foreach ($scormpopoupoptions as $name => $option) {
                 if (isset($scorm->$name)) {
                     $optionlist[] = $name.'='.$scorm->$name;
                 } else {
@@ -784,7 +766,7 @@ function scorm_reset_course_form_definition(&$mform) {
  * @return array
  */
 function scorm_reset_course_form_defaults($course) {
-    return array('reset_scorm'=>1);
+    return array('reset_scorm' => 1);
 }
 
 /**
@@ -831,15 +813,15 @@ function scorm_reset_userdata($data) {
 
         $DB->delete_records_select('scorm_scoes_track', "scormid IN ($scormssql)", array($data->courseid));
 
-        // remove all grades from gradebook
+        // Remove all grades from gradebook.
         if (empty($data->reset_gradebook_grades)) {
             scorm_reset_gradebook($data->courseid);
         }
 
-        $status[] = array('component'=>$componentstr, 'item'=>get_string('deleteallattempts', 'scorm'), 'error'=>false);
+        $status[] = array('component' => $componentstr, 'item' => get_string('deleteallattempts', 'scorm'), 'error' => false);
     }
 
-    // no dates to shift here
+    // No dates to shift here.
 
     return $status;
 }
@@ -891,7 +873,7 @@ function scorm_get_file_info($browser, $areas, $course, $cm, $context, $filearea
         return null;
     }
 
-    // no writing for now!
+    // No writing for now!
 
     $fs = get_file_storage();
 
@@ -905,7 +887,7 @@ function scorm_get_file_info($browser, $areas, $course, $cm, $context, $filearea
             if ($filepath === '/' and $filename === '.') {
                 $storedfile = new virtual_root_file($context->id, 'mod_scorm', 'content', 0);
             } else {
-                // not found
+                // Not found.
                 return null;
             }
         }
@@ -921,14 +903,14 @@ function scorm_get_file_info($browser, $areas, $course, $cm, $context, $filearea
             if ($filepath === '/' and $filename === '.') {
                 $storedfile = new virtual_root_file($context->id, 'mod_scorm', 'package', 0);
             } else {
-                // not found
+                // Not found.
                 return null;
             }
         }
         return new file_info_stored($browser, $context, $storedfile, $urlbase, $areas[$filearea], false, true, false, false);
     }
 
-    // scorm_intro handled in file_browser
+    // Scorm_intro handled in file_browser.
 
     return false;
 }
@@ -977,13 +959,15 @@ function scorm_pluginfile($course, $cm, $context, $filearea, $args, $forcedownlo
         // TODO: add any other access restrictions here if needed!
 
     } else if ($filearea === 'package') {
-        if (!$canmanageactivity) {
+        // Check if the global setting for disabling package downloads is enabled.
+        $protectpackagedownloads = get_config('scorm', 'protectpackagedownloads');
+        if ($protectpackagedownloads and !$canmanageactivity) {
             return false;
         }
         $revision = (int)array_shift($args); // Prevents caching problems - ignored here.
         $relativepath = implode('/', $args);
         $fullpath = "/$context->id/mod_scorm/package/0/$relativepath";
-        $lifetime = 0; // no caching here
+        $lifetime = 0; // No caching here.
 
     } else if ($filearea === 'imsmanifest') { // This isn't a real filearea, it's a url parameter for this type of package.
         $revision = (int)array_shift($args); // Prevents caching problems - ignored here.
@@ -1007,21 +991,20 @@ function scorm_pluginfile($course, $cm, $context, $filearea, $args, $forcedownlo
 
     $fs = get_file_storage();
     if (!$file = $fs->get_file_by_hash(sha1($fullpath)) or $file->is_directory()) {
-        if ($filearea === 'content') { //return file not found straight away to improve performance.
+        if ($filearea === 'content') { // Return file not found straight away to improve performance.
             send_header_404();
             die;
         }
         return false;
     }
 
-    // finally send the file
+    // Finally send the file.
     send_stored_file($file, $lifetime, 0, false, $options);
 }
 
 /**
  * @uses FEATURE_GROUPS
  * @uses FEATURE_GROUPINGS
- * @uses FEATURE_GROUPMEMBERSONLY
  * @uses FEATURE_MOD_INTRO
  * @uses FEATURE_COMPLETION_TRACKS_VIEWS
  * @uses FEATURE_COMPLETION_HAS_RULES
@@ -1034,7 +1017,6 @@ function scorm_supports($feature) {
     switch($feature) {
         case FEATURE_GROUPS:                  return false;
         case FEATURE_GROUPINGS:               return false;
-        case FEATURE_GROUPMEMBERSONLY:        return true;
         case FEATURE_MOD_INTRO:               return true;
         case FEATURE_COMPLETION_TRACKS_VIEWS: return true;
         case FEATURE_COMPLETION_HAS_RULES:    return true;
@@ -1133,19 +1115,17 @@ function scorm_print_overview($courses, &$htmlarray) {
             $showattemptstatus = true;
         }
         if ($showattemptstatus || !empty($isopen) || !empty($scorm->timeclose)) {
-            $str = '<div class="scorm overview"><div class="name">'.$strscorm. ': '.
-                '<a '.($scorm->visible ? '':' class="dimmed"').
-                'title="'.$strscorm.'" href="'.$CFG->wwwroot.
-                '/mod/scorm/view.php?id='.$scorm->coursemodule.'">'.
-                $scorm->name.'</a></div>';
+            $str = html_writer::start_div('scorm overview').html_writer::div($strscorm. ': '.
+                    html_writer::link($CFG->wwwroot.'/mod/scorm/view.php?id='.$scorm->coursemodule, $scorm->name,
+                                        array('title' => $strscorm, 'class' => $scorm->visible ? '' : 'dimmed')), 'name');
             if ($scorm->timeclose) {
-                $str .= '<div class="info">'.$strduedate.': '.userdate($scorm->timeclose).'</div>';
+                $str .= html_writer::div($strduedate.': '.userdate($scorm->timeclose), 'info');
             }
             if ($showattemptstatus) {
                 require_once($CFG->dirroot.'/mod/scorm/locallib.php');
-                $str .= '<div class="details">'.scorm_get_attempt_status($USER, $scorm).'</div>';
+                $str .= html_writer::div(scorm_get_attempt_status($USER, $scorm), 'details');
             }
-            $str .= '</div>';
+            $str .= html_writer::end_div();
             if (empty($htmlarray[$scorm->course]['scorm'])) {
                 $htmlarray[$scorm->course]['scorm'] = $str;
             } else {
@@ -1162,8 +1142,8 @@ function scorm_print_overview($courses, &$htmlarray) {
  * @param stdClass $currentcontext Current context of block
  */
 function scorm_page_type_list($pagetype, $parentcontext, $currentcontext) {
-    $module_pagetype = array('mod-scorm-*'=>get_string('page-mod-scorm-x', 'scorm'));
-    return $module_pagetype;
+    $modulepagetype = array('mod-scorm-*' => get_string('page-mod-scorm-x', 'scorm'));
+    return $modulepagetype;
 }
 
 /**
@@ -1174,7 +1154,7 @@ function scorm_page_type_list($pagetype, $parentcontext, $currentcontext) {
  */
 function scorm_version_check($scormversion, $version='') {
     $scormversion = trim(strtolower($scormversion));
-    if (empty($version) || $version==SCORM_12) {
+    if (empty($version) || $version == SCORM_12) {
         if ($scormversion == 'scorm_12' || $scormversion == 'scorm_1.2') {
             return SCORM_12;
         }
@@ -1217,7 +1197,7 @@ function scorm_get_completion_state($course, $cm, $userid, $type) {
 
     $result = $type;
 
-    // Get scorm
+    // Get scorm.
     if (!$scorm = $DB->get_record('scorm', array('id' => $cm->instance))) {
         print_error('cannotfindscorm');
     }
@@ -1254,10 +1234,10 @@ function scorm_get_completion_state($course, $cm, $userid, $type) {
         }
     }
 
-    // Check for status
+    // Check for status.
     if ($scorm->completionstatusrequired !== null) {
 
-        // Get status
+        // Get status.
         $statuses = array_flip(scorm_status_options());
         $nstatus = 0;
 
@@ -1279,7 +1259,7 @@ function scorm_get_completion_state($course, $cm, $userid, $type) {
 
     }
 
-    // Check for score
+    // Check for score.
     if ($scorm->completionscorerequired !== null) {
         $maxscore = -1;
 
@@ -1359,7 +1339,7 @@ function scorm_set_completion($scorm, $userid, $completionstate = COMPLETION_COM
     $course->id = $scorm->course;
     $completion = new completion_info($course);
 
-    // Check if completion is enabled site-wide, or for the course
+    // Check if completion is enabled site-wide, or for the course.
     if (!$completion->is_enabled()) {
         return;
     }
@@ -1369,7 +1349,7 @@ function scorm_set_completion($scorm, $userid, $completionstate = COMPLETION_COM
             return;
     }
 
-    if (empty($userid)) { //we need to get all the relevant users from $grades param.
+    if (empty($userid)) { // We need to get all the relevant users from $grades param.
         foreach ($grades as $grade) {
             $completion->update_state($cm, $completionstate, $grade->userid);
         }
@@ -1474,3 +1454,27 @@ function scorm_check_mode($scorm, &$newattempt, &$attempt, $userid, &$mode) {
         }
     }
 }
+
+/**
+ * Trigger the course_module_viewed event.
+ *
+ * @param  stdClass $scorm        scorm object
+ * @param  stdClass $course     course object
+ * @param  stdClass $cm         course module object
+ * @param  stdClass $context    context object
+ * @since Moodle 3.0
+ */
+function scorm_view($scorm, $course, $cm, $context) {
+
+    // Trigger course_module_viewed event.
+    $params = array(
+        'context' => $context,
+        'objectid' => $scorm->id
+    );
+
+    $event = \mod_scorm\event\course_module_viewed::create($params);
+    $event->add_record_snapshot('course_modules', $cm);
+    $event->add_record_snapshot('course', $course);
+    $event->add_record_snapshot('scorm', $scorm);
+    $event->trigger();
+}
