@@ -24,7 +24,7 @@
 
 defined('MOODLE_INTERNAL') || die();
 
-$plugin->version   = 2014051201;        // the current module version (YYYYMMDDXX)
-$plugin->requires  = 2014050800;        // requires this Moodle version
-$plugin->component = 'mod_workshop';    // full name of the plugin (used for diagnostics)
-$plugin->cron      = 60;                // give as a chance every minute
+$plugin->version   = 2015111600;        // The current module version (YYYYMMDDXX)
+$plugin->requires  = 2015111000;        // Requires this Moodle version.
+$plugin->component = 'mod_workshop';
+$plugin->cron      = 60;                // Give as a chance every minute.
